searchData={"items":[{"type":"module","title":"Postgrex","doc":"PostgreSQL driver for Elixir.\n\nPostgrex is a partial implementation of the Postgres [frontend/backend\nmessage protocol](https://www.postgresql.org/docs/current/protocol.html).\nIt performs wire messaging in Elixir, as opposed to binding to a library\nsuch as `libpq` in C.\n\nA Postgrex query is performed as \"[extended query](https://www.postgresql.org/docs/current/protocol-flow.html#PROTOCOL-FLOW-EXT-QUERY)\".\nAn \"extended query\"  involves separate server-side parse, bind, and execute\nstages, each of which may be re-used for efficiency. For example, libraries\nlike Ecto caches queries, so a query only has to be parsed and planned once.\nThis is all done via wire messaging, without relying on `PREPARE q AS (...)`\nand `EXECUTE q()` SQL statements directly.\n\nThis module handles the connection to PostgreSQL, providing support\nfor queries, transactions, connection backoff, logging, pooling and\nmore.\n\nNote that the notifications API (pub/sub) supported by PostgreSQL is\nhandled by `Postgrex.Notifications`. Hence, to use this feature,\nyou need to start a separate (notifications) connection.","ref":"Postgrex.html"},{"type":"function","title":"Postgrex.child_spec/1","doc":"Returns a supervisor child specification for a DBConnection pool.","ref":"Postgrex.html#child_spec/1"},{"type":"function","title":"Postgrex.close/3","doc":"Closes an (extended) prepared query and returns `:ok` or\n`{:error, %Postgrex.Error{}}` if there was an error. Closing a query releases\nany resources held by postgresql for a prepared query with that name. See\n`Postgrex.Query` for the query data.\n\nThis function may still raise an exception if there is an issue with types\n(`ArgumentError`), connection (`DBConnection.ConnectionError`), ownership\n(`DBConnection.OwnershipError`) or other error (`RuntimeError`).","ref":"Postgrex.html#close/3"},{"type":"function","title":"Options - Postgrex.close/3","doc":"* `:queue` - Whether to wait for connection in a queue (default: `true`);\n  * `:timeout` - Close request timeout (default: `15000`);\n  * `:mode` - set to `:savepoint` to use a savepoint to rollback to before the\n  close on error, otherwise set to `:transaction` (default: `:transaction`);","ref":"Postgrex.html#close/3-options"},{"type":"function","title":"Examples - Postgrex.close/3","doc":"query = Postgrex.prepare!(conn, \"\", \"CREATE TABLE posts (id serial, title text)\")\n    Postgrex.close(conn, query)","ref":"Postgrex.html#close/3-examples"},{"type":"function","title":"Postgrex.close!/3","doc":"Closes an (extended) prepared query and returns `:ok` or raises\n`Postgrex.Error` if there was an error. See `close/3`.","ref":"Postgrex.html#close!/3"},{"type":"function","title":"Postgrex.execute/4","doc":"Runs an (extended) prepared query.\n\nIt returns the result as `{:ok, %Postgrex.Query{}, %Postgrex.Result{}}` or\n`{:error, %Postgrex.Error{}}` if there was an error. Parameters are given as\npart of the prepared query, `%Postgrex.Query{}`.\n\nSee the README for information on how Postgrex encodes and decodes Elixir\nvalues by default. See `Postgrex.Query` for the query data and\n`Postgrex.Result` for the result data.","ref":"Postgrex.html#execute/4"},{"type":"function","title":"Options - Postgrex.execute/4","doc":"* `:queue` - Whether to wait for connection in a queue (default: `true`);\n  * `:timeout` - Execute request timeout (default: `15000`);\n  * `:decode_mapper` - Fun to map each row in the result to a term after\n  decoding, (default: `fn x -> x end`);\n  * `:mode` - set to `:savepoint` to use a savepoint to rollback to before the\n  execute on error, otherwise set to `:transaction` (default: `:transaction`);","ref":"Postgrex.html#execute/4-options"},{"type":"function","title":"Examples - Postgrex.execute/4","doc":"query = Postgrex.prepare!(conn, \"\", \"CREATE TABLE posts (id serial, title text)\")\n    Postgrex.execute(conn, query, [])\n\n    query = Postgrex.prepare!(conn, \"\", \"SELECT id FROM posts WHERE title like $1\")\n    Postgrex.execute(conn, query, [\"%my%\"])","ref":"Postgrex.html#execute/4-examples"},{"type":"function","title":"Postgrex.execute!/4","doc":"Runs an (extended) prepared query and returns the result or raises\n`Postgrex.Error` if there was an error. See `execute/4`.","ref":"Postgrex.html#execute!/4"},{"type":"function","title":"Postgrex.parameters/2","doc":"Returns a cached map of connection parameters.","ref":"Postgrex.html#parameters/2"},{"type":"function","title":"Options - Postgrex.parameters/2","doc":"* `:timeout` - Call timeout (default: `15000`)","ref":"Postgrex.html#parameters/2-options"},{"type":"function","title":"Postgrex.prepare/4","doc":"Prepares an (extended) query.\n\nIt returns the result as `{:ok, %Postgrex.Query{}}` or `{:error, %Postgrex.Error{}}`\nif there was an error. Parameters can be set in the query as `$1` embedded in the\nquery string. To execute the query call `execute/4`. To close the prepared query\ncall `close/3`. See `Postgrex.Query` for the query data.\n\nThis function may still raise an exception if there is an issue with types\n(`ArgumentError`), connection (`DBConnection.ConnectionError`), ownership\n(`DBConnection.OwnershipError`) or other error (`RuntimeError`).\n\nFor unnamed prepared statements, pass an empty string for `name`. This can be useful\nwhen trying to avoid the generic query plan Postgres creates for named prepared\nstatements. You may also set `prepare: :unnamed` at the connection level so that\nevery prepared statement using that connection will be unnamed.","ref":"Postgrex.html#prepare/4"},{"type":"function","title":"Options - Postgrex.prepare/4","doc":"* `:queue` - Whether to wait for connection in a queue (default: `true`);\n  * `:timeout` - Prepare request timeout (default: `15000`);\n  * `:mode` - set to `:savepoint` to use a savepoint to rollback to before the\n  prepare on error, otherwise set to `:transaction` (default: `:transaction`);","ref":"Postgrex.html#prepare/4-options"},{"type":"function","title":"Examples - Postgrex.prepare/4","doc":"Postgrex.prepare(conn, \"\", \"CREATE TABLE posts (id serial, title text)\")","ref":"Postgrex.html#prepare/4-examples"},{"type":"function","title":"Postgrex.prepare!/4","doc":"Prepares an (extended) query and returns the prepared query or raises\n`Postgrex.Error` if there was an error. See `prepare/4`.","ref":"Postgrex.html#prepare!/4"},{"type":"function","title":"Postgrex.prepare_execute/5","doc":"Prepares and executes a query in a single step.\n\nIt returns the result as `{:ok, %Postgrex.Query{}, %Postgrex.Result{}}` or\n`{:error, %Postgrex.Error{}}` if there was an error. Parameters are given as\npart of the prepared query, `%Postgrex.Query{}`.\n\nFor unnamed prepared statements, pass an empty string for `name`. This can be useful\nwhen trying to avoid the generic query plan Postgres creates for named prepared\nstatements. You may also set `prepare: :unnamed` at the connection level so that\nevery prepared statement using that connection will be unnamed.\n\nSee the README for information on how Postgrex encodes and decodes Elixir\nvalues by default. See `Postgrex.Query` for the query data and\n`Postgrex.Result` for the result data.","ref":"Postgrex.html#prepare_execute/5"},{"type":"function","title":"Options - Postgrex.prepare_execute/5","doc":"* `:queue` - Whether to wait for connection in a queue (default: `true`);\n  * `:timeout` - Execute request timeout (default: `15000`);\n  * `:decode_mapper` - Fun to map each row in the result to a term after\n  decoding, (default: `fn x -> x end`);\n  * `:mode` - set to `:savepoint` to use a savepoint to rollback to before the\n  execute on error, otherwise set to `:transaction` (default: `:transaction`);","ref":"Postgrex.html#prepare_execute/5-options"},{"type":"function","title":"Examples - Postgrex.prepare_execute/5","doc":"Postgrex.prepare_execute(conn, \"\", \"SELECT id FROM posts WHERE title like $1\", [\"%my%\"])","ref":"Postgrex.html#prepare_execute/5-examples"},{"type":"function","title":"Postgrex.prepare_execute!/5","doc":"Prepares and runs a query and returns the result or raises\n`Postgrex.Error` if there was an error. See `prepare_execute/5`.","ref":"Postgrex.html#prepare_execute!/5"},{"type":"function","title":"Postgrex.query/4","doc":"Runs an (extended) query and returns the result as `{:ok, %Postgrex.Result{}}`\nor `{:error, %Postgrex.Error{}}` if there was a database error. Parameters can\nbe set in the query as `$1` embedded in the query string. Parameters are given\nas a list of elixir values. See the README for information on how Postgrex\nencodes and decodes Elixir values by default. See `Postgrex.Result` for the\nresult data.\n\nThis function may still raise an exception if there is an issue with types\n(`ArgumentError`), connection (`DBConnection.ConnectionError`), ownership\n(`DBConnection.OwnershipError`) or other error (`RuntimeError`).","ref":"Postgrex.html#query/4"},{"type":"function","title":"Options - Postgrex.query/4","doc":"* `:queue` - Whether to wait for connection in a queue (default: `true`);\n  * `:timeout` - Query request timeout (default: `15000`);\n  * `:decode_mapper` - Fun to map each row in the result to a term after\n  decoding, (default: `fn x -> x end`);\n  * `:mode` - set to `:savepoint` to use a savepoint to rollback to before the\n  query on error, otherwise set to `:transaction` (default: `:transaction`);\n  * `:cache_statement` - Caches the query with the given name","ref":"Postgrex.html#query/4-options"},{"type":"function","title":"Examples - Postgrex.query/4","doc":"Postgrex.query(conn, \"CREATE TABLE posts (id serial, title text)\", [])\n\n    Postgrex.query(conn, \"INSERT INTO posts (title) VALUES ('my title')\", [])\n\n    Postgrex.query(conn, \"SELECT title FROM posts\", [])\n\n    Postgrex.query(conn, \"SELECT id FROM posts WHERE title like $1\", [\"%my%\"])\n\n    Postgrex.query(conn, \"COPY posts TO STDOUT\", [])","ref":"Postgrex.html#query/4-examples"},{"type":"function","title":"Postgrex.query!/4","doc":"Runs an (extended) query and returns the result or raises `Postgrex.Error` if\nthere was an error. See `query/3`.","ref":"Postgrex.html#query!/4"},{"type":"function","title":"Postgrex.rollback/2","doc":"Rollback a transaction, does not return.\n\nAborts the current transaction fun. If inside multiple `transaction/3`\nfunctions, bubbles up to the top level.","ref":"Postgrex.html#rollback/2"},{"type":"function","title":"Example - Postgrex.rollback/2","doc":"{:error, :oops} = Postgrex.transaction(pid, fn(conn) ->\n      DBConnection.rollback(conn, :bar)\n      IO.puts \"never reaches here!\"\n    end)","ref":"Postgrex.html#rollback/2-example"},{"type":"function","title":"Postgrex.start_link/1","doc":"Start the connection process and connect to postgres.","ref":"Postgrex.html#start_link/1"},{"type":"function","title":"Options - Postgrex.start_link/1","doc":"Postgrex provides multiple ways to connect to the server, listed in order of\nprecedence below:\n\n  * `:hostname` - Server hostname (default: PGHOST env variable, then localhost);\n  * `:port` - Server port (default: PGPORT env variable, then 5432);\n  * `:endpoints` - A list of endpoints (host and port pairs, with an optional\n    extra_opts keyword list);\n    Postgrex will try each endpoint in order, one by one, until the connection succeeds;\n    The syntax is `[{host1, port1},{host2, port2},{host3, port3}]` or\n    `[{host1, port1, extra_opt1: value},{host2, port2, extra_opt2: value}}]`;\n    This option takes precedence over `:hostname+:port`;\n  * `:socket_dir` - Connect to PostgreSQL via UNIX sockets in the given directory;\n    The socket name is derived based on the port. This is the preferred method\n    for configuring sockets and it takes precedence over the hostname. If you are\n    connecting to a socket outside of the PostgreSQL convention, use `:socket` instead;\n  * `:socket` - Connect to PostgreSQL via UNIX sockets in the given path.\n    This option takes precedence over the `:hostname`, `:endpoints` and `:socket_dir`;\n\nOnce a server is specified, you can configure the connection with the following:\n\n  * `:database` - Database (default: PGDATABASE env variable; otherwise required);\n\n  * `:username` - Username (default: PGUSER env variable, then USER env var);\n\n  * `:password` - User password (default: PGPASSWORD env variable);\n\n  * `:parameters` - Keyword list of connection parameters;\n\n  * `:timeout` - Socket receive timeout when idle in milliseconds (default:\n  `15000`);\n\n  * `:connect_timeout` - Socket connect timeout in milliseconds (defaults to\n    `:timeout` value);\n\n  * `:handshake_timeout` - Connection handshake timeout in milliseconds\n    (defaults to `:timeout` value);\n\n  * `:ping_timeout` - Socket receive timeout when idle in milliseconds (defaults to\n    `:timeout` value);\n\n  * `:idle_interval` - Ping connections after a period of inactivity in milliseconds.\n    Defaults to 1000ms;\n\n  * `:ssl` - Enables SSL. Setting it to `true` enables SSL without server certificate verification,\n    which emits a warning. Instead, prefer to set it to a keyword list, with either\n    `:cacerts` or `:cacertfile` set to a CA trust store, to enable server certificate\n    verification. Defaults to `false`;\n\n  * `:socket_options` - Options to be given to the underlying socket\n    (applies to both TCP and UNIX sockets);\n\n  * `:target_server_type` - Allows opening connections to a server in the given\n    replica mode. The allowed values are `:any`, `:primary` and `:secondary`\n    (default: `:any`). If this option is used together with `endpoints`, we will\n    traverse all endpoints until we find an endpoint matching the server type;\n\n  * `:disconnect_on_error_codes` - List of error code atoms that when encountered\n    will disconnect the connection. This is useful when using Postgrex against systems that\n    support failover, which when it occurs will emit certain error codes\n    e.g. `:read_only_sql_transaction` (default: `[]`);\n\n  * `:show_sensitive_data_on_connection_error` - By default, `Postgrex`\n    hides all information during connection errors to avoid leaking credentials\n    or other sensitive information. You can set this option if you wish to\n    see complete errors and stacktraces during connection errors;\n\nThe following options controls the pool and other Postgrex features:\n\n  * `:prepare` - How to prepare queries, either `:named` to use named queries\n  or `:unnamed` to force unnamed queries (default: `:named`);\n\n  * `:transactions` - Set to `:strict` to error on unexpected transaction\n    state, otherwise set to `:naive` (default: `:strict`);\n\n  * `:pool` - The pool module to use, defaults to `DBConnection.ConnectionPool`.\n    See the pool documentation for more options. The default `:pool_size` for\n    the default pool is 1. If you set a different pool, this option must be\n    included with all requests contacting the pool;\n\n  * `:types` - The types module to use, see `Postgrex.Types.define/3`, this\n    option is only required when using custom encoding or decoding (default:\n    `Postgrex.DefaultTypes`);\n\n  * `:search_path` - A list of strings used to set the search path for the connection.\n    This is useful when, for instance, an extension like `citext` is installed in a\n    separate schema. If that schema is not in the connection's search path, Postgrex\n    might not be able to recognize the extension's data type. When this option is `nil`,\n    the search path is not modified. (default: `nil`).\n    See the [PostgreSQL docs](https://www.postgresql.org/docs/current/ddl-schemas.html#DDL-SCHEMAS-PATH)\n    for more details.\n\n  * `:disable_composite_types` - Set to `true` to disable composite types support.\n    This is useful when using Postgrex against systems that do not support composite types\n    (default: `false`).\n\n`Postgrex` uses the `DBConnection` library and supports all `DBConnection`\noptions like `:idle`, `:after_connect` etc. See `DBConnection.start_link/2`\nfor more information.","ref":"Postgrex.html#start_link/1-options"},{"type":"function","title":"Examples - Postgrex.start_link/1","doc":"iex> {:ok, pid} = Postgrex.start_link(database: \"postgres\")\n    {:ok, #PID<0.69.0>}\n\nRun a query after connection has been established:\n\n    iex> {:ok, pid} = Postgrex.start_link(after_connect: &Postgrex.query!(&1, \"SET TIME ZONE 'UTC';\", []))\n    {:ok, #PID<0.69.0>}\n\nConnect to postgres instance through a unix domain socket\n\n    iex> {:ok, pid} = Postgrex.start_link(socket_dir: \"/tmp\", database: \"postgres\")\n    {:ok, #PID<0.69.0>}","ref":"Postgrex.html#start_link/1-examples"},{"type":"function","title":"SSL client authentication - Postgrex.start_link/1","doc":"When connecting to Postgres or CockroachDB instances over SSL it is idiomatic to use\ncertificate authentication:\n\n    [ssl: [cacertfile: System.get_env(\"DB_CA_CERT_FILE\")]]\n\nThe server name indication (SNI) will be automatically set based on the `:hostname`\nconfiguration, if one was provided. Other options, such as `depth: 3`, may be necessary\ndepending on the server.","ref":"Postgrex.html#start_link/1-ssl-client-authentication"},{"type":"function","title":"PgBouncer - Postgrex.start_link/1","doc":"When using PgBouncer with transaction or statement pooling named prepared\nqueries can not be used because the bouncer may route requests from\nthe same postgrex connection to different PostgreSQL backend processes\nand discards named queries after the transactions closes.\nTo force unnamed prepared queries set the `:prepare` option to `:unnamed`.","ref":"Postgrex.html#start_link/1-pgbouncer"},{"type":"function","title":"Handling failover - Postgrex.start_link/1","doc":"Some services, such as AWS Aurora, support failovers. The 2 options\n`endpoints` and `target_server_type` can be used together to achieve a faster fail-over.\n\nImagine an AWS Aurora cluster named \"test\" with 2 instances. Use the\nfollowing options minimize downtime by ensuring that Postgrex connects to\nthe new primary instance as soon as possible.\n\n    {:ok, pid} = Postgrex.start_link(\n      endpoints: [\n        {\"test.cluster-xyz.eu-west-1.rds.amazonaws.com\", 5432},\n        {\"test.cluster-ro-xyz.eu-west-1.rds.amazonaws.com\", 5432}\n      ],\n      target_server_type: :primary,\n      (...)\n    )\n\nIn the event of a fail-over, Postgrex gets first disconnected from what used\nto be the primary instance. The primary instance will then reboot and turn into\na secondary instance. Meanwhile, one of the secondary instances will have turned\ninto the new primary instance. However, the DNS entry of the primary endpoint\nprovided by AWS can take some time to get updated. That is why it can be faster to\nlet Postgrex iterate over all the instances of the cluster to find the new\nprimary instance instead of waiting for the DNS update.\n\nIf the cluster does not have DNS-backed primary and secondary endpoints (like the\nones provided by AWS Aurora) or if the cluster is made of more than 2 instances,\nthe hostname (and port) of all of the individual instances can be specified\nin the `endpoints` list:\n\n    endpoints: [\n      {\"test-instance-1.xyz.eu-west-1.rds.amazonaws.com\", 5432},\n      {\"test-instance-2.xyz.eu-west-1.rds.amazonaws.com\", 5432},\n      (...),\n      {\"test-instance-N.xyz.eu-west-1.rds.amazonaws.com\", 5432}\n    ]","ref":"Postgrex.html#start_link/1-handling-failover"},{"type":"function","title":"Postgrex.stream/4","doc":"Returns a stream for a query on a connection.\n\nStream consumes memory in chunks of at most `max_rows` rows (see Options).\nThis is useful for processing _large_ datasets.\n\nA stream must be wrapped in a transaction and may be used as an `Enumerable`\nor a `Collectable`.\n\nWhen used as an `Enumerable` with a `COPY .. TO STDOUT` SQL query no other\nqueries or streams can be interspersed until the copy has finished. Otherwise\nit is possible to intersperse enumerable streams and queries.\n\nWhen used as a `Collectable` the values are passed as copy data with the\nquery. No other queries or streams can be interspersed until the copy has\nfinished. If the query is not copying to the database the copy data will still\nbe sent but is silently discarded.","ref":"Postgrex.html#stream/4"},{"type":"function","title":"Options - Postgrex.stream/4","doc":"* `:max_rows` - Maximum numbers of rows in a result (default to `500`)\n  * `:decode_mapper` - Fun to map each row in the result to a term after\n  decoding, (default: `fn x -> x end`);\n  * `:mode` - set to `:savepoint` to use a savepoint to rollback to before an\n  execute on error, otherwise set to `:transaction` (default: `:transaction`);","ref":"Postgrex.html#stream/4-options"},{"type":"function","title":"Examples - Postgrex.stream/4","doc":"Postgrex.transaction(pid, fn(conn) ->\n      query = Postgrex.prepare!(conn, \"\", \"COPY posts TO STDOUT\")\n      stream = Postgrex.stream(conn, query, [])\n      result_to_iodata = fn(%Postgrex.Result{rows: rows}) -> rows end\n      Enum.into(stream, File.stream!(\"posts\"), result_to_iodata)\n    end)\n\n    Postgrex.transaction(pid, fn(conn) ->\n      stream = Postgrex.stream(conn, \"COPY posts FROM STDIN\", [])\n      Enum.into(File.stream!(\"posts\"), stream)\n    end)","ref":"Postgrex.html#stream/4-examples"},{"type":"function","title":"Postgrex.transaction/3","doc":"Acquire a lock on a connection and run a series of requests inside a\ntransaction. The result of the transaction fun is return inside an `:ok`\ntuple: `{:ok, result}`.\n\nTo use the locked connection call the request with the connection\nreference passed as the single argument to the `fun`. If the\nconnection disconnects all future calls using that connection\nreference will fail.\n\n`rollback/2` rolls back the transaction and causes the function to\nreturn `{:error, reason}`.\n\n`transaction/3` can be nested multiple times if the connection\nreference is used to start a nested transaction. The top level\ntransaction function is the actual transaction.","ref":"Postgrex.html#transaction/3"},{"type":"function","title":"Options - Postgrex.transaction/3","doc":"* `:queue` - Whether to wait for connection in a queue (default: `true`);\n  * `:timeout` - Transaction timeout (default: `15000`);\n  * `:mode` - Set to `:savepoint` to use savepoints instead of an SQL\n  transaction, otherwise set to `:transaction` (default: `:transaction`);\n  * `:commit_comment` - When a binary string is provided, prepends the text as\n  a comment attached to the `COMMIT` statement issued to close the transaction (default: `nil`);\n\nThe `:timeout` is for the duration of the transaction and all nested\ntransactions and requests. This timeout overrides timeouts set by internal\ntransactions and requests. The `:mode` will be used for all requests inside\nthe transaction function. The `:commit_comment` can be helpful in distinguishing\nbetween transactions in query performance monitoring tools.","ref":"Postgrex.html#transaction/3-options"},{"type":"function","title":"Example - Postgrex.transaction/3","doc":"{:ok, res} = Postgrex.transaction(pid, fn(conn) ->\n      Postgrex.query!(conn, \"SELECT title FROM posts\", [])\n    end)","ref":"Postgrex.html#transaction/3-example"},{"type":"type","title":"Postgrex.conn/0","doc":"A connection process name, pid or reference.\n\nA connection reference is used when making multiple requests to the same\nconnection, see `transaction/3`.","ref":"Postgrex.html#t:conn/0"},{"type":"type","title":"Postgrex.execute_option/0","doc":"","ref":"Postgrex.html#t:execute_option/0"},{"type":"type","title":"Postgrex.option/0","doc":"","ref":"Postgrex.html#t:option/0"},{"type":"type","title":"Postgrex.start_option/0","doc":"","ref":"Postgrex.html#t:start_option/0"},{"type":"module","title":"Postgrex.Extensions.Multirange","doc":"","ref":"Postgrex.Extensions.Multirange.html"},{"type":"function","title":"Postgrex.Extensions.Multirange.decode/1","doc":"","ref":"Postgrex.Extensions.Multirange.html#decode/1"},{"type":"function","title":"Postgrex.Extensions.Multirange.decode/3","doc":"","ref":"Postgrex.Extensions.Multirange.html#decode/3"},{"type":"function","title":"Postgrex.Extensions.Multirange.encode/1","doc":"","ref":"Postgrex.Extensions.Multirange.html#encode/1"},{"type":"function","title":"Postgrex.Extensions.Multirange.encode/2","doc":"","ref":"Postgrex.Extensions.Multirange.html#encode/2"},{"type":"function","title":"Postgrex.Extensions.Multirange.format/1","doc":"","ref":"Postgrex.Extensions.Multirange.html#format/1"},{"type":"function","title":"Postgrex.Extensions.Multirange.init/1","doc":"","ref":"Postgrex.Extensions.Multirange.html#init/1"},{"type":"function","title":"Postgrex.Extensions.Multirange.matching/1","doc":"","ref":"Postgrex.Extensions.Multirange.html#matching/1"},{"type":"function","title":"Postgrex.Extensions.Multirange.oids/2","doc":"","ref":"Postgrex.Extensions.Multirange.html#oids/2"},{"type":"module","title":"Postgrex.Multirange","doc":"Struct for PostgreSQL `multirange`.","ref":"Postgrex.Multirange.html"},{"type":"module","title":"Fields - Postgrex.Multirange","doc":"* `ranges`","ref":"Postgrex.Multirange.html#module-fields"},{"type":"type","title":"Postgrex.Multirange.t/0","doc":"","ref":"Postgrex.Multirange.html#t:t/0"},{"type":"module","title":"Postgrex.Notifications","doc":"API for notifications (pub/sub) in PostgreSQL.\n\nIn order to use it, first you need to start the notification process.\nIn your supervision tree:\n\n    {Postgrex.Notifications, name: MyApp.Notifications}\n\nThen you can listen to certain channels:\n\n    {:ok, listen_ref} = Postgrex.Notifications.listen(MyApp.Notifications, \"channel\")\n\nNow every time a message is broadcast on said channel, for example via\nPostgreSQL command line:\n\n    NOTIFY \"channel\", \"Oh hai!\";\n\nYou will receive a message in the format:\n\n    {:notification, notification_pid, listen_ref, channel, message}","ref":"Postgrex.Notifications.html"},{"type":"module","title":"Async connect, auto-reconnects and missed notifications - Postgrex.Notifications","doc":"By default, the notification system establishes a connection to the\ndatabase on initialization, you can configure the connection to happen\nasynchronously. You can also configure the connection to automatically\nreconnect.\n\nNote however that when the notification system is waiting for a connection,\nany notifications that occur during the disconnection period are not queued\nand cannot be recovered. Similarly, any listen command will be queued until\nthe connection is up.\n\nThere is a race condition between starting to listen and notifications being\nissued \"at the same time\", as explained [in the PostgreSQL documentation](https://www.postgresql.org/docs/current/sql-listen.html).\nIf your application needs to keep a consistent representation of data, follow\nthe three-step approach of first subscribing, then obtaining the current\nstate of data, then handling the incoming notifications.\n\nBeware that the same\nrace condition applies to auto-reconnects. A simple way of dealing with this\nissue is not using the auto-reconnect feature directly, but monitoring and\nre-starting the Notifications process, then subscribing to channel messages\nover again, using the same three-step approach.","ref":"Postgrex.Notifications.html#module-async-connect-auto-reconnects-and-missed-notifications"},{"type":"module","title":"A note on casing - Postgrex.Notifications","doc":"While PostgreSQL seems to behave as case-insensitive, it actually has a very\npeculiar behaviour on casing. When you write:\n\n    SELECT * FROM POSTS\n\nPostgreSQL actually converts `POSTS` into the lowercase `posts`. That's why\nboth `SELECT * FROM POSTS` and `SELECT * FROM posts` feel equivalent.\nHowever, if you wrap the table name in quotes, then the casing in quotes\nwill be preserved.\n\nThese same rules apply to PostgreSQL notification channels. More importantly,\nwhenever `Postgrex.Notifications` listens to a channel, it wraps the channel\nname in quotes. Therefore, if you listen to a channel named \"fooBar\" and\nyou send a notification without quotes in the channel name, such as:\n\n    NOTIFY fooBar, \"Oh hai!\";\n\nThe notification will not be received by Postgrex.Notifications because the\nnotification will be effectively sent to `\"foobar\"` and not `\"fooBar\"`. Therefore,\nyou must guarantee one of the two following properties:\n\n  1. If you can wrap the channel name in quotes when sending a notification,\n     then make sure the channel name has the exact same casing when listening\n     and sending notifications\n\n  2. If you cannot wrap the channel name in quotes when sending a notification,\n     then make sure to give the lowercased channel name when listening","ref":"Postgrex.Notifications.html#module-a-note-on-casing"},{"type":"function","title":"Postgrex.Notifications.listen/3","doc":"Listens to an asynchronous notification channel using the `LISTEN` command.\n\nA message `{:notification, connection_pid, ref, channel, payload}` will be\nsent to the calling process when a notification is received.\n\nIt returns `{:ok, reference}`. It may also return `{:eventually, reference}`\nif the notification process is not currently connected to the database and\nit was started with `:sync_connect` set to false or `:auto_reconnect` set\nto true. The `reference` can be used to issue an `unlisten/3` command.","ref":"Postgrex.Notifications.html#listen/3"},{"type":"function","title":"Options - Postgrex.Notifications.listen/3","doc":"* `:timeout` - Call timeout (default: `5000`)","ref":"Postgrex.Notifications.html#listen/3-options"},{"type":"function","title":"Postgrex.Notifications.listen!/3","doc":"Listens to an asynchronous notification channel `channel`. See `listen/2`.","ref":"Postgrex.Notifications.html#listen!/3"},{"type":"function","title":"Postgrex.Notifications.start_link/1","doc":"Start the notification connection process and connect to postgres.\n\nThe options that this function accepts are the same as those accepted by\n`Postgrex.start_link/1`, as well as the extra options `:sync_connect`,\n`:auto_reconnect`, `:reconnect_backoff`, and `:configure`.","ref":"Postgrex.Notifications.html#start_link/1"},{"type":"function","title":"Options - Postgrex.Notifications.start_link/1","doc":"* `:sync_connect` - controls if the connection should be established on boot\n    or asynchronously right after boot. Defaults to `true`.\n\n  * `:auto_reconnect` - automatically attempt to reconnect to the database\n    in event of a disconnection. See the\n    [note about async connect and auto-reconnects](#module-async-connect-and-auto-reconnects)\n    above. Defaults to `false`, which means the process terminates.\n\n  * `:reconnect_backoff` - time (in ms) between reconnection attempts when\n    `auto_reconnect` is enabled. Defaults to `500`.\n\n  * `:idle_interval` - while also accepted on `Postgrex.start_link/1`, it has\n    a default of `5000ms` in `Postgrex.Notifications` (instead of 1000ms).\n\n  * `:configure` - A function to run before every connect attempt to dynamically\n    configure the options as a `{module, function, args}`, where the current\n    options will prepended to `args`. Defaults to `nil`.","ref":"Postgrex.Notifications.html#start_link/1-options"},{"type":"function","title":"Postgrex.Notifications.unlisten/3","doc":"Stops listening on the given channel by passing the reference returned from\n`listen/2`.","ref":"Postgrex.Notifications.html#unlisten/3"},{"type":"function","title":"Options - Postgrex.Notifications.unlisten/3","doc":"* `:timeout` - Call timeout (default: `5000`)","ref":"Postgrex.Notifications.html#unlisten/3-options"},{"type":"function","title":"Postgrex.Notifications.unlisten!/3","doc":"Stops listening on the given channel by passing the reference returned from\n`listen/2`.","ref":"Postgrex.Notifications.html#unlisten!/3"},{"type":"type","title":"Postgrex.Notifications.server/0","doc":"","ref":"Postgrex.Notifications.html#t:server/0"},{"type":"module","title":"Postgrex.Query","doc":"Query struct returned from a successfully prepared query.\n\nIts public fields are:\n\n  * `name` - The name of the prepared statement;\n  * `statement` - The prepared statement;\n  * `columns` - The column names;\n  * `ref` - A reference used to identify prepared queries;","ref":"Postgrex.Query.html"},{"type":"module","title":"Prepared queries - Postgrex.Query","doc":"Once a query is prepared with `Postgrex.prepare/4`, the\nreturned query will have its `ref` field set to a reference.\nWhen `Postgrex.execute/4` is called with the prepared query,\nit always returns a query. If the `ref` field in the query\ngiven to `execute` and the one returned are the same, it\nmeans the cached prepared query was used. If the `ref` field\nis not the same, it means the query had to be re-prepared.","ref":"Postgrex.Query.html#module-prepared-queries"},{"type":"type","title":"Postgrex.Query.t/0","doc":"","ref":"Postgrex.Query.html#t:t/0"},{"type":"behaviour","title":"Postgrex.ReplicationConnection","doc":"A process that receives and sends PostgreSQL replication messages.\n\n> Note: this module is experimental and may be subject to changes\n> in the future.","ref":"Postgrex.ReplicationConnection.html"},{"type":"behaviour","title":"Logical replication - Postgrex.ReplicationConnection","doc":"Let's see how to use this module for connecting to PostgreSQL\nfor logical replication. First of all, you need to configure the\nwal level in PostgreSQL to logical. Run this inside your PostgreSQL\nshell/configuration:\n\n    ALTER SYSTEM SET wal_level='logical';\n    ALTER SYSTEM SET max_wal_senders='10';\n    ALTER SYSTEM SET max_replication_slots='10';\n\nThen **you must restart your server**. Alternatively, you can set\nthose values when starting \"postgres\". This is useful, for example,\nwhen running it from Docker:\n\n    services:\n      postgres:\n        image: postgres:14\n        env:\n          ...\n        command: [\"postgres\", \"-c\", \"wal_level=logical\"]\n\nFor CI, GitHub Actions do not support setting command, so you can\nupdate and restart Postgres instead in a step:\n\n    - name: \"Set PG settings\"\n      run: |\n        docker exec ${{ job.services.postgres.id }} sh -c 'echo \"wal_level=logical\" >> /var/lib/postgresql/data/postgresql.conf'\n        docker restart ${{ job.services.pg.id }}\n\nThen you must create a publication to be replicated.\nThis can be done in any session:\n\n    CREATE PUBLICATION postgrex_example FOR ALL TABLES;\n\nYou can also filter if you want to publish insert, update,\ndelete or a subset of them:\n\n    # Skips updates (keeps inserts, deletes, begins, commits, etc)\n    create PUBLICATION postgrex_example FOR ALL TABLES WITH (publish = 'insert,delete');\n\n    # Skips inserts, updates, and deletes (keeps begins, commits, etc)\n    create PUBLICATION postgrex_example FOR ALL TABLES WITH (publish = '');\n\nNow we are ready to create module that starts a replication slot\nand listens to our publication. Our example will use the pgoutput\nfor logical replication and print all incoming messages to the\nterminal:\n\n    Mix.install([:postgrex])\n\n    defmodule Repl do\n      use Postgrex.ReplicationConnection\n\n      def start_link(opts) do\n        # Automatically reconnect if we lose connection.\n        extra_opts = [\n          auto_reconnect: true\n        ]\n\n        Postgrex.ReplicationConnection.start_link(__MODULE__, :ok, extra_opts ++ opts)\n      end\n\n      @impl true\n      def init(:ok) do\n        {:ok, %{step: :disconnected}}\n      end\n\n      @impl true\n      def handle_connect(state) do\n        query = \"CREATE_REPLICATION_SLOT postgrex TEMPORARY LOGICAL pgoutput NOEXPORT_SNAPSHOT\"\n        {:query, query, %{state | step: :create_slot}}\n      end\n\n      @impl true\n      def handle_result(results, %{step: :create_slot} = state) when is_list(results) do\n        query = \"START_REPLICATION SLOT postgrex LOGICAL 0/0 (proto_version '1', publication_names 'postgrex_example')\"\n        {:stream, query, [], %{state | step: :streaming}}\n      end\n\n      @impl true\n      # https://www.postgresql.org/docs/14/protocol-replication.html\n      def handle_data(<<?w, _wal_start::64, _wal_end::64, _clock::64, rest::binary>>, state) do\n        IO.inspect(rest)\n        {:noreply, state}\n      end\n\n      def handle_data(<<?k, wal_end::64, _clock::64, reply>>, state) do\n        messages =\n          case reply do\n            1 -> [<<?r, wal_end + 1::64, wal_end + 1::64, wal_end + 1::64, current_time()::64, 0>>]\n            0 -> []\n          end\n\n        {:noreply, messages, state}\n      end\n\n      @epoch DateTime.to_unix(~U[2000-01-01 00:00:00Z], :microsecond)\n      defp current_time(), do: System.os_time(:microsecond) - @epoch\n    end\n\n    {:ok, pid} =\n      Repl.start_link(\n        host: \"localhost\",\n        database: \"demo_dev\",\n        username: \"postgres\",\n      )\n\n    Process.sleep(:infinity)\n\n## `use` options\n\n`use Postgrex.ReplicationConnection` accepts a list of options which configures the\nchild specification and therefore how it runs under a supervisor.\nThe generated `child_spec/1` can be customized with the following options:\n\n  * `:id` - the child specification identifier, defaults to the current module\n  * `:restart` - when the child should be restarted, defaults to `:permanent`\n  * `:shutdown` - how to shut down the child, either immediately or by giving\n    it time to shut down\n\nFor example:\n\n    use Postgrex.ReplicationConnection, restart: :transient, shutdown: 10_000\n\nSee the \"Child specification\" section in the `Supervisor` module for more\ndetailed information. The `@doc` annotation immediately preceding\n`use Postgrex.ReplicationConnection` will be attached to the generated `child_spec/1`\nfunction.","ref":"Postgrex.ReplicationConnection.html#module-logical-replication"},{"type":"behaviour","title":"Name registration - Postgrex.ReplicationConnection","doc":"A `Postgrex.ReplicationConnection` is bound to the same name registration rules as a\n`GenServer`. Read more about them in the `GenServer` docs.","ref":"Postgrex.ReplicationConnection.html#module-name-registration"},{"type":"function","title":"Postgrex.ReplicationConnection.call/3","doc":"Calls the given replication server.","ref":"Postgrex.ReplicationConnection.html#call/3"},{"type":"function","title":"Postgrex.ReplicationConnection.decode_lsn/1","doc":"Returns the integer representation of an LSN value, given its string representation.\n\nIt returns `:error` if the provided string is not a valid LSN.","ref":"Postgrex.ReplicationConnection.html#decode_lsn/1"},{"type":"function","title":"Log Sequence Numbers - Postgrex.ReplicationConnection.decode_lsn/1","doc":"PostgreSQL uses two representations for the Log Sequence Number (LSN):\n\n  * An unsigned 64-bit integer. Used internally by PostgreSQL and sent in the XLogData\n  replication messages.\n\n  * A string of two hexadecimal numbers of up to eight digits each, separated\n  by a slash. e.g. `1/F73E0220`. This is the form accepted by `start_replication/2`.\n\nFor more information on Log Sequence Numbers, see\n[PostgreSQL pg_lsn docs](https://www.postgresql.org/docs/current/datatype-pg-lsn.html) and\n[PostgreSQL WAL internals docs](https://www.postgresql.org/docs/current/wal-internals.html).","ref":"Postgrex.ReplicationConnection.html#decode_lsn/1-log-sequence-numbers"},{"type":"function","title":"Postgrex.ReplicationConnection.encode_lsn/1","doc":"Returns the string representation of an LSN value, given its integer representation.\n\nIt returns `:error` if the provided integer falls outside the range for a valid\nunsigned 64-bit integer.","ref":"Postgrex.ReplicationConnection.html#encode_lsn/1"},{"type":"function","title":"Log Sequence Numbers - Postgrex.ReplicationConnection.encode_lsn/1","doc":"PostgreSQL uses two representations for the Log Sequence Number (LSN):\n\n  * An unsigned 64-bit integer. Used internally by PostgreSQL and sent in the XLogData\n  replication messages.\n\n  * A string of two hexadecimal numbers of up to eight digits each, separated\n  by a slash. e.g. `1/F73E0220`. This is the form accepted by `start_replication/2`.\n\nFor more information on Log Sequence Numbers, see\n[PostgreSQL pg_lsn docs](https://www.postgresql.org/docs/current/datatype-pg-lsn.html) and\n[PostgreSQL WAL internals docs](https://www.postgresql.org/docs/current/wal-internals.html).","ref":"Postgrex.ReplicationConnection.html#encode_lsn/1-log-sequence-numbers"},{"type":"callback","title":"Postgrex.ReplicationConnection.handle_call/3","doc":"Callback for `call/3`.\n\nReplies must be sent with `reply/2`.\n\nIf `auto_reconnect: false` (the default) and there is a disconnection,\nthe process will terminate and the caller will exit even if no reply is\nsent. However, if `auto_reconnect` is set to true, a disconnection will\nkeep the process alive, which means that any command that has not yet\nbeen replied to should eventually do so. One simple approach is to\nreply to any pending commands on `c:handle_disconnect/1`.","ref":"Postgrex.ReplicationConnection.html#c:handle_call/3"},{"type":"callback","title":"Postgrex.ReplicationConnection.handle_connect/1","doc":"Invoked after connecting.\n\nThis may be invoked multiple times if `:auto_reconnect` is set to true.","ref":"Postgrex.ReplicationConnection.html#c:handle_connect/1"},{"type":"callback","title":"Postgrex.ReplicationConnection.handle_data/2","doc":"Callback for `:stream` outputs.\n\nIf any callback returns `{:stream, iodata, opts, state}`, then this\ncallback will be eventually called with the result of the query.\nIt receives `binary` streaming messages.\n\nThis can be useful for replication and copy commands. For replication,\nthe format of the messages are described [under the START_REPLICATION\nsection in PostgreSQL docs](https://www.postgresql.org/docs/14/protocol-replication.html).\nReplication messages may require explicit acknowledgement, which can\nbe done by returning a list of binaries according to the replication\nprotocol.","ref":"Postgrex.ReplicationConnection.html#c:handle_data/2"},{"type":"callback","title":"Postgrex.ReplicationConnection.handle_disconnect/1","doc":"Invoked after disconnecting.\n\nThis is only invoked if `:auto_reconnect` is set to true.","ref":"Postgrex.ReplicationConnection.html#c:handle_disconnect/1"},{"type":"callback","title":"Postgrex.ReplicationConnection.handle_info/2","doc":"Callback for `Kernel.send/2`.","ref":"Postgrex.ReplicationConnection.html#c:handle_info/2"},{"type":"callback","title":"Postgrex.ReplicationConnection.handle_result/2","doc":"Callback for `:query` outputs.\n\nIf any callback returns `{:query, iodata, state}`,\nthen this callback will be immediately called with\nthe result of the query. Please note that even though\nreplication connections use the simple query protocol,\nPostgres currently limits them to single command queries.\nDue to this constraint, this callback will be passed\neither a list with a single successful query result or\nan error.","ref":"Postgrex.ReplicationConnection.html#c:handle_result/2"},{"type":"callback","title":"Postgrex.ReplicationConnection.init/1","doc":"Callback for process initialization.\n\nThis is called once and before the Postgrex connection is established.","ref":"Postgrex.ReplicationConnection.html#c:init/1"},{"type":"function","title":"Postgrex.ReplicationConnection.reply/2","doc":"Replies to the given `call/3`.","ref":"Postgrex.ReplicationConnection.html#reply/2"},{"type":"function","title":"Postgrex.ReplicationConnection.start_link/3","doc":"Starts a replication process with the given callback `module`.","ref":"Postgrex.ReplicationConnection.html#start_link/3"},{"type":"function","title":"Options - Postgrex.ReplicationConnection.start_link/3","doc":"The options that this function accepts are the same as those\naccepted by `Postgrex.start_link/1`, except for `:idle_interval`.\n\nIt also accepts extra options for connection management, documented below.\nAlso note this function also automatically set `:replication` to `\"database\"`\nas part of the connection `:parameters` if none is set yet.","ref":"Postgrex.ReplicationConnection.html#start_link/3-options"},{"type":"function","title":"Connection options - Postgrex.ReplicationConnection.start_link/3","doc":"* `:sync_connect` - controls if the connection should be established on boot\n    or asynchronously right after boot. Defaults to `true`.\n\n  * `:auto_reconnect` - automatically attempt to reconnect to the database\n    in event of a disconnection. See the\n    [note about async connect and auto-reconnects](#module-async-connect-and-auto-reconnects)\n    above. Defaults to `false`, which means the process terminates.\n\n  * `:reconnect_backoff` - time (in ms) between reconnection attempts when\n    `:auto_reconnect` is enabled. Defaults to `500`.","ref":"Postgrex.ReplicationConnection.html#start_link/3-connection-options"},{"type":"type","title":"Postgrex.ReplicationConnection.ack/0","doc":"","ref":"Postgrex.ReplicationConnection.html#t:ack/0"},{"type":"type","title":"Postgrex.ReplicationConnection.query/0","doc":"","ref":"Postgrex.ReplicationConnection.html#t:query/0"},{"type":"type","title":"Postgrex.ReplicationConnection.reason/0","doc":"","ref":"Postgrex.ReplicationConnection.html#t:reason/0"},{"type":"type","title":"Postgrex.ReplicationConnection.server/0","doc":"","ref":"Postgrex.ReplicationConnection.html#t:server/0"},{"type":"type","title":"Postgrex.ReplicationConnection.state/0","doc":"","ref":"Postgrex.ReplicationConnection.html#t:state/0"},{"type":"type","title":"Postgrex.ReplicationConnection.stream_opts/0","doc":"The following options configure streaming:\n\n  * `:max_messages` - The maximum number of replications messages that can be\n    accumulated from the wire until they are relayed to `handle_data/2`.\n    Defaults to `500`.","ref":"Postgrex.ReplicationConnection.html#t:stream_opts/0"},{"type":"module","title":"Postgrex.Result","doc":"Result struct returned from any successful query. Its fields are:\n\n  * `command` - An atom or a list of atoms of the query command, for example:\n    `:select`, `:insert`, or `[:rollback, :release]`;\n  * `columns` - The column names;\n  * `rows` - The result set. A list of lists, each inner list corresponding to a\n    row, each element in the inner list corresponds to a column;\n  * `num_rows` - The number of fetched or affected rows;\n  * `connection_id` - The OS pid of the PostgreSQL backend that executed the query;\n  * `messages` - A list of maps of messages, such as hints and notices, sent by the\n    driver during the execution of the query.","ref":"Postgrex.Result.html"},{"type":"type","title":"Postgrex.Result.t/0","doc":"","ref":"Postgrex.Result.html#t:t/0"},{"type":"behaviour","title":"Postgrex.SimpleConnection","doc":"A generic connection suitable for simple queries and pubsub functionality.\n\nOn its own, a SimpleConnection server only maintains a connection. To execute\nqueries, process results, or relay notices you must implement a callback module\nwith the SimpleConnection behaviour.","ref":"Postgrex.SimpleConnection.html"},{"type":"behaviour","title":"Example - Postgrex.SimpleConnection","doc":"The SimpleConnection behaviour abstracts common client/server interactions,\nalong with optional mechanisms for running queries or relaying notifications.\n\nLet's start with a minimal callback module that executes a query and relays\nthe result back to the caller.\n\n    defmodule MyConnection do\n      @behaviour Postgrex.SimpleConnection\n\n      @impl true\n      def init(_args) do\n        {:ok, %{from: nil}}\n      end\n\n      @impl true\n      def handle_call({:query, query}, from, state) do\n        {:query, query, %{state | from: from}}\n      end\n\n      @impl true\n      def handle_result(results, state) when is_list(results) do\n        SimpleConnection.reply(state.from, results)\n\n        {:noreply, state}\n      end\n\n      @impl true\n      def handle_result(%Postgrex.Error{} = error, state) do\n        SimpleConnection.reply(state.from, error)\n\n        {:noreply, state}\n      end\n    end\n\n    # Start the connection\n    {:ok, pid} = SimpleConnection.start_link(MyConnection, [], database: \"demo\")\n\n    # Execute a literal query\n    SimpleConnection.call(pid, {:query, \"SELECT 1\"})\n    # => %Postgrex.Result{rows: [[\"1\"]]}\n\nWe start a connection by passing the callback module, callback options, and\nserver options to `SimpleConnection.start_link/3`. The `init/1` function\nreceives any callback options and returns the callback state.\n\nQueries are sent through `SimpleConnection.call/2`, executed on the server,\nand the result is handed off to `handle_result/2`. At that point the callback\ncan process the result before replying back to the caller with\n`SimpleConnection.reply/2`.","ref":"Postgrex.SimpleConnection.html#module-example"},{"type":"behaviour","title":"Building a PubSub Connection - Postgrex.SimpleConnection","doc":"With the `notify/3` callback you can also build a pubsub server on top of\n`LISTEN/NOTIFY`. Here's a naive pubsub implementation:\n\n    defmodule MyPubSub do\n      @behaviour Postgrex.SimpleConnection\n\n      defstruct [:from, listeners: %{}]\n\n      @impl true\n      def init(args) do\n        {:ok, struct!(__MODULE__, args)}\n      end\n\n      @impl true\n      def notify(channel, payload, state) do\n        for pid <- state.listeners[channel] do\n          send(pid, {:notice, channel, payload})\n        end\n      end\n\n      @impl true\n      def handle_call({:listen, channel}, {pid, _} = from, state) do\n        listeners = Map.update(state.listeners, channel, [pid], &[pid | &1])\n\n        {:query, ~s(LISTEN \"#{channel}\"), %{state | from: from, listeners: listeners}}\n      end\n\n      def handle_call({:query, query}, from, state) do\n        {:query, query, %{state | from: from}}\n      end\n\n      @impl true\n      def handle_result(_results, state) do\n        SimpleConnection.reply(state.from, :ok)\n\n        {:noreply, %{state | from: nil}}\n      end\n    end\n\n    # Start the connection\n    {:ok, pid} = SimpleConnection.start_link(MyPubSub, [], database: \"demo\")\n\n    # Start listening to the \"demo\" channel\n    SimpleConnection.call(pid, {:listen, \"demo\"})\n    # => %Postgrex.Result{command: :listen}\n\n    # Notify all listeners\n    SimpleConnection.call(pid, {:query, ~s(NOTIFY \"demo\", 'hello')})\n    # => %Postgrex.Result{command: :notify}\n\n    # Check the inbox to see the notice message\n    flush()\n    # => {:notice, \"demo\", \"hello\"}\n\nSee `Postgrex.Notifications` for a more complex implementation that can\nunlisten, handle process exits, and persist across reconnection.","ref":"Postgrex.SimpleConnection.html#module-building-a-pubsub-connection"},{"type":"behaviour","title":"Name registration - Postgrex.SimpleConnection","doc":"A `Postgrex.SimpleConnection` is bound to the same name registration rules as a\n`GenServer`. Read more about them in the `GenServer` docs.","ref":"Postgrex.SimpleConnection.html#module-name-registration"},{"type":"function","title":"Postgrex.SimpleConnection.call/3","doc":"Calls the given server.\n\nWrapper for `:gen_statem.call/3`.","ref":"Postgrex.SimpleConnection.html#call/3"},{"type":"callback","title":"Postgrex.SimpleConnection.handle_call/3","doc":"Callback for `SimpleConnection.call/3`.\n\nReplies must be sent with `SimpleConnection.reply/2`.","ref":"Postgrex.SimpleConnection.html#c:handle_call/3"},{"type":"callback","title":"Postgrex.SimpleConnection.handle_connect/1","doc":"Invoked after connecting or reconnecting.\n\nThis may be called multiple times if `:auto_reconnect` is true.","ref":"Postgrex.SimpleConnection.html#c:handle_connect/1"},{"type":"callback","title":"Postgrex.SimpleConnection.handle_disconnect/1","doc":"Invoked after disconnection.\n\nThis is invoked regardless of the `:auto_reconnect` option.","ref":"Postgrex.SimpleConnection.html#c:handle_disconnect/1"},{"type":"callback","title":"Postgrex.SimpleConnection.handle_info/2","doc":"Callback for `Kernel.send/2`.","ref":"Postgrex.SimpleConnection.html#c:handle_info/2"},{"type":"callback","title":"Postgrex.SimpleConnection.handle_result/2","doc":"Callback for processing or relaying queries executed via `{:query, query, state}`.\n\nEither a list of successful query results or an error will be passed to this callback.\nA list is passed because the simple query protocol allows multiple commands to be\nissued in a single query.","ref":"Postgrex.SimpleConnection.html#c:handle_result/2"},{"type":"callback","title":"Postgrex.SimpleConnection.init/1","doc":"Callback for process initialization.\n\nThis is called once and before the Postgrex connection is established.","ref":"Postgrex.SimpleConnection.html#c:init/1"},{"type":"callback","title":"Postgrex.SimpleConnection.notify/3","doc":"Callback for processing or relaying pubsub notifications.","ref":"Postgrex.SimpleConnection.html#c:notify/3"},{"type":"function","title":"Postgrex.SimpleConnection.reply/2","doc":"Replies to the given client.\n\nWrapper for `:gen_statem.reply/2`.","ref":"Postgrex.SimpleConnection.html#reply/2"},{"type":"function","title":"Postgrex.SimpleConnection.start_link/3","doc":"Start the connection process and connect to Postgres.\n\nThe options that this function accepts are the same as those accepted by\n`Postgrex.start_link/1`, as well as the extra options `:sync_connect`,\n`:auto_reconnect`, `:reconnect_backoff`, and `:configure`.","ref":"Postgrex.SimpleConnection.html#start_link/3"},{"type":"function","title":"Options - Postgrex.SimpleConnection.start_link/3","doc":"* `:auto_reconnect` - automatically attempt to reconnect to the database\n    in event of a disconnection. See the\n    [note about async connect and auto-reconnects](#module-async-connect-and-auto-reconnects)\n    above. Defaults to `false`, which means the process terminates.\n\n  * `:configure` - A function to run before every connect attempt to dynamically\n    configure the options as a `{module, function, args}`, where the current\n    options will prepended to `args`. Defaults to `nil`.\n\n  * `:idle_interval` - while also accepted on `Postgrex.start_link/1`, it has\n    a default of `5000ms` in `Postgrex.SimpleConnection` (instead of 1000ms).\n\n  * `:reconnect_backoff` - time (in ms) between reconnection attempts when\n    `auto_reconnect` is enabled. Defaults to `500`.\n\n  * `:sync_connect` - controls if the connection should be established on boot\n    or asynchronously right after boot. Defaults to `true`.","ref":"Postgrex.SimpleConnection.html#start_link/3-options"},{"type":"type","title":"Postgrex.SimpleConnection.from/0","doc":"","ref":"Postgrex.SimpleConnection.html#t:from/0"},{"type":"type","title":"Postgrex.SimpleConnection.query/0","doc":"","ref":"Postgrex.SimpleConnection.html#t:query/0"},{"type":"type","title":"Postgrex.SimpleConnection.state/0","doc":"","ref":"Postgrex.SimpleConnection.html#t:state/0"},{"type":"module","title":"Postgrex.Stream","doc":"Stream struct returned from stream commands.\n\nAll of its fields are private.","ref":"Postgrex.Stream.html"},{"type":"type","title":"Postgrex.Stream.t/0","doc":"","ref":"Postgrex.Stream.html#t:t/0"},{"type":"module","title":"Postgrex.Box","doc":"Struct for PostgreSQL `box`.","ref":"Postgrex.Box.html"},{"type":"module","title":"Fields - Postgrex.Box","doc":"* `upper_right`\n  * `bottom_left`","ref":"Postgrex.Box.html#module-fields"},{"type":"type","title":"Postgrex.Box.t/0","doc":"","ref":"Postgrex.Box.html#t:t/0"},{"type":"module","title":"Postgrex.Circle","doc":"Struct for PostgreSQL `circle`.","ref":"Postgrex.Circle.html"},{"type":"module","title":"Fields - Postgrex.Circle","doc":"* `center`\n  * `radius`","ref":"Postgrex.Circle.html#module-fields"},{"type":"type","title":"Postgrex.Circle.t/0","doc":"","ref":"Postgrex.Circle.html#t:t/0"},{"type":"module","title":"Postgrex.INET","doc":"Struct for PostgreSQL `inet` / `cidr`.","ref":"Postgrex.INET.html"},{"type":"module","title":"Fields - Postgrex.INET","doc":"* `address`\n  * `netmask`","ref":"Postgrex.INET.html#module-fields"},{"type":"type","title":"Postgrex.INET.t/0","doc":"","ref":"Postgrex.INET.html#t:t/0"},{"type":"module","title":"Postgrex.Interval","doc":"Struct for PostgreSQL `interval`.","ref":"Postgrex.Interval.html"},{"type":"module","title":"Fields - Postgrex.Interval","doc":"* `months`\n  * `days`\n  * `secs`\n  * `microsecs`","ref":"Postgrex.Interval.html#module-fields"},{"type":"function","title":"Postgrex.Interval.compare/2","doc":"","ref":"Postgrex.Interval.html#compare/2"},{"type":"function","title":"Postgrex.Interval.to_string/1","doc":"","ref":"Postgrex.Interval.html#to_string/1"},{"type":"type","title":"Postgrex.Interval.t/0","doc":"","ref":"Postgrex.Interval.html#t:t/0"},{"type":"module","title":"Postgrex.Lexeme","doc":"Struct for PostgreSQL `lexeme`.","ref":"Postgrex.Lexeme.html"},{"type":"module","title":"Fields - Postgrex.Lexeme","doc":"* `word`\n  * `positions`","ref":"Postgrex.Lexeme.html#module-fields"},{"type":"type","title":"Postgrex.Lexeme.t/0","doc":"","ref":"Postgrex.Lexeme.html#t:t/0"},{"type":"module","title":"Postgrex.Line","doc":"Struct for PostgreSQL `line`.\n\nNote, lines are stored in PostgreSQL in the form `{a, b, c}`, which\nparameterizes a line as `a*x + b*y + c = 0`.","ref":"Postgrex.Line.html"},{"type":"module","title":"Fields - Postgrex.Line","doc":"* `a`\n  * `b`\n  * `c`","ref":"Postgrex.Line.html#module-fields"},{"type":"type","title":"Postgrex.Line.t/0","doc":"","ref":"Postgrex.Line.html#t:t/0"},{"type":"module","title":"Postgrex.LineSegment","doc":"Struct for PostgreSQL `lseg`.","ref":"Postgrex.LineSegment.html"},{"type":"module","title":"Fields - Postgrex.LineSegment","doc":"* `point1`\n  * `point2`","ref":"Postgrex.LineSegment.html#module-fields"},{"type":"type","title":"Postgrex.LineSegment.t/0","doc":"","ref":"Postgrex.LineSegment.html#t:t/0"},{"type":"module","title":"Postgrex.MACADDR","doc":"Struct for PostgreSQL `macaddr`.","ref":"Postgrex.MACADDR.html"},{"type":"module","title":"Fields - Postgrex.MACADDR","doc":"* `address`","ref":"Postgrex.MACADDR.html#module-fields"},{"type":"type","title":"Postgrex.MACADDR.macaddr/0","doc":"","ref":"Postgrex.MACADDR.html#t:macaddr/0"},{"type":"type","title":"Postgrex.MACADDR.t/0","doc":"","ref":"Postgrex.MACADDR.html#t:t/0"},{"type":"module","title":"Postgrex.Path","doc":"Struct for PostgreSQL `path`.","ref":"Postgrex.Path.html"},{"type":"module","title":"Fields - Postgrex.Path","doc":"* `open`\n  * `points`","ref":"Postgrex.Path.html#module-fields"},{"type":"type","title":"Postgrex.Path.t/0","doc":"","ref":"Postgrex.Path.html#t:t/0"},{"type":"module","title":"Postgrex.Point","doc":"Struct for PostgreSQL `point`.","ref":"Postgrex.Point.html"},{"type":"module","title":"Fields - Postgrex.Point","doc":"* `x`\n  * `y`","ref":"Postgrex.Point.html#module-fields"},{"type":"type","title":"Postgrex.Point.t/0","doc":"","ref":"Postgrex.Point.html#t:t/0"},{"type":"module","title":"Postgrex.Polygon","doc":"Struct for PostgreSQL `polygon`.","ref":"Postgrex.Polygon.html"},{"type":"module","title":"Fields - Postgrex.Polygon","doc":"* `vertices`","ref":"Postgrex.Polygon.html#module-fields"},{"type":"type","title":"Postgrex.Polygon.t/0","doc":"","ref":"Postgrex.Polygon.html#t:t/0"},{"type":"module","title":"Postgrex.Range","doc":"Struct for PostgreSQL `range`.\n\nNote that PostgreSQL itself does not return ranges exactly as stored:\n`SELECT '(1,5)'::int4range` returns `[2,5)`, which is equivalent in terms\nof the values included in the range ([PostgreSQL docs](https://www.postgresql.org/docs/current/rangetypes.html#RANGETYPES-IO)).\nWhen selecting data, this struct simply reflects what PostgreSQL returns.","ref":"Postgrex.Range.html"},{"type":"module","title":"Fields - Postgrex.Range","doc":"* `lower`\n  * `upper`\n  * `lower_inclusive`\n  * `upper_inclusive`","ref":"Postgrex.Range.html#module-fields"},{"type":"type","title":"Postgrex.Range.t/0","doc":"","ref":"Postgrex.Range.html#t:t/0"},{"type":"module","title":"Postgrex.DefaultTypes","doc":"The default module used to encode/decode PostgreSQL types.\n\nType modules are given as the `:types` option in `Postgrex.start_link/1`.","ref":"Postgrex.DefaultTypes.html"},{"type":"behaviour","title":"Postgrex.Extension","doc":"An extension knows how to encode and decode PostgreSQL types to and\nfrom Elixir values.\n\nCustom extensions can be enabled via `Postgrex.Types.define/3`.\n`Postgrex.Types.define/3` must be called on its own file, outside of\nany module and function, as it only needs to be defined once during\ncompilation.\n\nFor example to support label trees using the text encoding format:\n\n    defmodule MyApp.LTree do\n      @behaviour Postgrex.Extension\n\n      # It can be memory efficient to copy the decoded binary because a\n      # reference counted binary that points to a larger binary will be passed\n      # to the decode/4 callback. Copying the binary can allow the larger\n      # binary to be garbage collected sooner if the copy is going to be kept\n      # for a longer period of time. See `:binary.copy/1` for more\n      # information.\n      def init(opts) do\n        Keyword.get(opts, :decode_copy, :copy)\n      end\n\n      # Use this extension when `type` from %Postgrex.TypeInfo{} is \"ltree\"\n      def matching(_state), do: [type: \"ltree\"]\n\n      # Use the text format, \"ltree\" does not have a binary format.\n      def format(_state), do: :text\n\n      # Use quoted expression to encode a string that is the same as\n      # postgresql's ltree text format. The quoted expression should contain\n      # clauses that match those of a `case` or `fn`. Encoding matches on the\n      # value and returns encoded `iodata()`. The first 4 bytes in the\n      # `iodata()` must be the byte size of the rest of the encoded data, as a\n      # signed 32bit big endian integer.\n      def encode(_state) do\n        quote do\n          bin when is_binary(bin) ->\n            [< > | bin]\n        end\n      end\n\n      # Use quoted expression to decode the data to a string. Decoding matches\n      # on an encoded binary with the same signed 32bit big endian integer\n      # length header.\n      def decode(:reference) do\n        quote do\n          < > ->\n            bin\n        end\n      end\n      def decode(:copy) do\n        quote do\n          < > ->\n            :binary.copy(bin)\n        end\n      end\n    end\n\nThis example could be used in a custom types module:\n\n    Postgrex.Types.define(MyApp.Types, [MyApp.LTree])\n\nOr pass in opts for the extension that will be passed to the `init/1` callback:\n\n    Postgrex.Types.define(MyApp.Types, [{MyApp.LTree, [decode_copy: :copy]}])","ref":"Postgrex.Extension.html"},{"type":"callback","title":"Postgrex.Extension.decode/1","doc":"Returns a quoted list of clauses that decode a binary to an Elixir value.\n\nThe pattern must use binary syntax and decode a fixed length using the signed\n32 bit big endian integer byte length header.\n\n    def decode(_) do\n      quote do\n        # length header is in bytes\n        < > ->\n          integer\n      end\n    end","ref":"Postgrex.Extension.html#c:decode/1"},{"type":"callback","title":"Postgrex.Extension.encode/1","doc":"Returns a quoted list of clauses that encode an Elixir value to iodata.\n\nIt must use a signed 32 bit big endian integer byte length header.\n\n    def encode(_) do\n      quote do\n        integer ->\n          <<8::signed-32, integer::signed-64>>\n      end\n    end","ref":"Postgrex.Extension.html#c:encode/1"},{"type":"callback","title":"Postgrex.Extension.format/1","doc":"Returns the format the type should be encoded as. See\nhttp://www.postgresql.org/docs/9.4/static/protocol-overview.html#PROTOCOL-FORMAT-CODES.","ref":"Postgrex.Extension.html#c:format/1"},{"type":"callback","title":"Postgrex.Extension.init/1","doc":"Should perform any initialization of the extension. The function receives the\nuser options. The state returned from this function will be passed to other\ncallbacks.","ref":"Postgrex.Extension.html#c:init/1"},{"type":"callback","title":"Postgrex.Extension.matching/1","doc":"Specifies the types the extension matches, see `Postgrex.TypeInfo` for\nspecification of the fields.","ref":"Postgrex.Extension.html#c:matching/1"},{"type":"callback","title":"Postgrex.Extension.prelude/1","doc":"Prelude defines properties and values that are attached to the body of\nthe types module.","ref":"Postgrex.Extension.html#c:prelude/1"},{"type":"type","title":"Postgrex.Extension.state/0","doc":"","ref":"Postgrex.Extension.html#t:state/0"},{"type":"type","title":"Postgrex.Extension.t/0","doc":"","ref":"Postgrex.Extension.html#t:t/0"},{"type":"module","title":"Postgrex.TypeInfo","doc":"The information about a type that is provided to the custom encoder/decoder\nfunctions. See http://www.postgresql.org/docs/9.4/static/catalog-pg-type.html\nfor clarifications of the fields.\n\n  * `oid` - The type's id;\n  * `type` - The type name;\n  * `send` - The name of the \"send\" function (the function postgres uses\n    to convert the type to its binary format);\n  * `receive` - The name of the \"receive\" function (the function postgres uses\n    to convert the type from its binary format);\n  * `output` - The name of the \"output\" function (the function postgres uses\n    to convert the type to its text format);\n  * `input` - The name of the \"input\" function (the function postgres uses\n    to convert the type from its text format);\n  * `array_elem` - If this is an array, the array elements' oid;\n  * `base_type` - If this is a range type, the base type's oid;\n  * `comp_elems` - If this is a composite type (record), the tuple\n    elements' oid;","ref":"Postgrex.TypeInfo.html"},{"type":"type","title":"Postgrex.TypeInfo.t/0","doc":"","ref":"Postgrex.TypeInfo.html#t:t/0"},{"type":"module","title":"Postgrex.Types","doc":"Encodes and decodes between PostgreSQL protocol and Elixir values.","ref":"Postgrex.Types.html"},{"type":"function","title":"Postgrex.Types.define/3","doc":"Defines a type module with custom extensions and options.\n\n`Postgrex.Types.define/3` must be called on its own file, outside of\nany module and function, as it only needs to be defined once during\ncompilation.\n\nType modules are given to Postgrex on `start_link` via the `:types`\noption and are used to control how Postgrex encodes and decodes data\ncoming from PostgreSQL.\n\nFor example, to define a new type module with a custom extension\ncalled `MyExtension` while also changing `Postgrex`'s default\nbehaviour regarding binary decoding, you may create a new file\ncalled \"lib/my_app/postgrex_types.ex\" with the following:\n\n    Postgrex.Types.define(MyApp.PostgrexTypes, [MyExtension], [decode_binary: :reference])\n\nThe line above will define a new module, called `MyApp.PostgrexTypes`\nwhich can be passed as `:types` when starting Postgrex. The type module\nworks by rewriting and inlining the extensions' encode and decode\nexpressions in an optimal fashion for postgrex to encode parameters and\ndecode multiple rows at a time.","ref":"Postgrex.Types.html#define/3"},{"type":"function","title":"Extensions - Postgrex.Types.define/3","doc":"Extensions is a list of `Postgrex.Extension` modules or a 2-tuple\ncontaining the module and a keyword list. The keyword, defaulting\nto `[]`, will be passed to the modules `init/1` callback.\n\nExtensions at the front of the list will take priority over later\nextensions when the `matching/1` callback returns have conflicting\nmatches. If an extension is not provided for a type then Postgrex\nwill fallback to default encoding/decoding methods where possible.\nAll extensions that ship as part of Postgrex are included out of the\nbox.\n\nSee `Postgrex.Extension` for more information on extensions.","ref":"Postgrex.Types.html#define/3-extensions"},{"type":"function","title":"Options - Postgrex.Types.define/3","doc":"* `:null` - The atom to use as a stand in for postgres' `NULL` in\n    encoding and decoding. The module attribute `@null` is registered\n    with the value so that extension can access the value if desired\n    (default: `nil`);\n\n  * `:decode_binary` - Either `:copy` to copy binary values when decoding\n    with default extensions that return binaries or `:reference` to use a\n    reference counted binary of the binary received from the socket.\n    Referencing a potentially larger binary can be more efficient if the binary\n    value is going to be garbaged collected soon because a copy is avoided.\n    However the larger binary can not be garbage collected until all references\n    are garbage collected (default: `:copy`);\n\n  * `:json` - The JSON module to encode and decode JSON binaries, calls\n    `module.encode_to_iodata!/1` to encode and `module.decode!/1` to decode.\n    If `nil` then no default JSON handling\n    (default: `Application.get_env(:postgrex, :json_library, Jason)`);\n\n  * `:bin_opt_info` - Either `true` to enable binary optimisation information,\n    or `false` to disable, for more information see `Kernel.SpecialForms.<<>>/1`\n    in Elixir (default: `false`);\n\n  * `:debug_defaults` - Generate debug information when building default\n    extensions so they point to the proper source. Enabling such option\n    will increase the time to compile the type module (default: `false`);\n\n  * `:moduledoc` - The moduledoc to be used for the generated module.\n\n  * `:allow_infinite_timestamps` - A boolean controlling whether or not\n    the built-in extensions `timestamp` and `timestamptz` will allow\n    a value of infinity to be decoded. Defaults to `false`.\n\n  * `:interval_decode_type` - The struct that intervals will be decoded\n    into. Either `Postgrex.Interval` or `Duration` (Elixir 1.17.0+ only).\n    Defaults to `Postgrex.Interval`.","ref":"Postgrex.Types.html#define/3-options"},{"type":"type","title":"Postgrex.Types.oid/0","doc":"PostgreSQL internal identifier that maps to a type. See\n .","ref":"Postgrex.Types.html#t:oid/0"},{"type":"opaque","title":"Postgrex.Types.state/0","doc":"State used by the encoder/decoder functions","ref":"Postgrex.Types.html#t:state/0"},{"type":"opaque","title":"Postgrex.Types.type/0","doc":"Term used to describe type information","ref":"Postgrex.Types.html#t:type/0"},{"type":"exception","title":"Postgrex.Error","doc":"","ref":"Postgrex.Error.html"},{"type":"type","title":"Postgrex.Error.t/0","doc":"","ref":"Postgrex.Error.html#t:t/0"},{"type":"exception","title":"Postgrex.QueryError","doc":"","ref":"Postgrex.QueryError.html"},{"type":"extras","title":"Postgrex","doc":"# Postgrex\n\n[![Build Status](https://github.com/elixir-ecto/postgrex/workflows/CI/badge.svg)](https://github.com/elixir-ecto/postgrex/actions)\n\nPostgreSQL driver for Elixir.\n\nDocumentation: http://hexdocs.pm/postgrex/","ref":"readme.html"},{"type":"extras","title":"Examples - Postgrex","doc":"```iex\niex> {:ok, pid} = Postgrex.start_link(hostname: \"localhost\", username: \"postgres\", password: \"postgres\", database: \"postgres\")\n{:ok, #PID<0.69.0>}\n\niex> Postgrex.query!(pid, \"SELECT user_id, text FROM comments\", [])\n%Postgrex.Result{command: :select, empty?: false, columns: [\"user_id\", \"text\"], rows: [[3,\"hey\"],[4,\"there\"]], size: 2}}\n\niex> Postgrex.query!(pid, \"INSERT INTO comments (user_id, text) VALUES (10, 'heya')\", [])\n%Postgrex.Result{command: :insert, columns: nil, rows: nil, num_rows: 1}}\n```","ref":"readme.html#examples"},{"type":"extras","title":"Features - Postgrex","doc":"* Automatic decoding and encoding of Elixir values to and from PostgreSQL's binary format\n  * User defined extensions for encoding and decoding any PostgreSQL type\n  * Supports transactions, prepared queries and multiple pools via [DBConnection](https://github.com/elixir-ecto/db_connection)\n  * Supports PostgreSQL 8.4, 9.0-9.6, and later (hstore is not supported on 8.4)","ref":"readme.html#features"},{"type":"extras","title":"Data representation - Postgrex","doc":"| PostgreSQL         | Elixir                                                                                                                                      |\n|--------------------|---------------------------------------------------------------------------------------------------------------------------------------------|\n| `NULL`             | `nil`                                                                                                                                       |\n| `bool`             | `true`, `false`                                                                                                                             |\n| `char`             | `\"é\"`                                                                                                                                       |\n| `int`              | `42`                                                                                                                                        |\n| `float`            | `42.0`                                                                                                                                      |\n| `text`             | `\"eric\"`                                                                                                                                    |\n| `bytea`            | `<<42>>`                                                                                                                                    |\n| `numeric`          | `#Decimal<42.0>` (1)                                                                                                                        |\n| `date`             | `%Date{year: 2013, month: 10, day: 12}`                                                                                                     |\n| `time(tz)`         | `%Time{hour: 0, minute: 37, second: 14}` (2)                                                                                                |\n| `timestamp`        | `%NaiveDateTime{year: 2013, month: 10, day: 12, hour: 0, minute: 37, second: 14}`                                                           |\n| `timestamptz`      | `%DateTime{year: 2013, month: 10, day: 12, hour: 0, minute: 37, second: 14, time_zone: \"Etc/UTC\"}` (2)                                      |\n| `interval`         | `%Postgrex.Interval{months: 14, days: 40, secs: 10920, microsecs: 315}`                                                                     |\n| `interval`         | `%Duration{year: 1, month: 2, week: 5, day: 5, hour: 3, minute: 2, second: 0, microsecond: {315, 6}}` (3)                                   |\n| `array`            | `[1, 2, 3]`                                                                                                                                 |\n| `composite type`   | `{42, \"title\", \"content\"}`                                                                                                                  |\n| `range`            | `%Postgrex.Range{lower: 1, upper: 5}`                                                                                                       |\n| `multirange`       | `%Postgrex.Multirange{ranges: [%Postgrex.Range{lower: 1, upper: 5}, %Postgrex.Range{lower: 20, upper: 23}]}`                                |\n| `uuid`             | `<<160,238,188,153,156,11,78,248,187,109,107,185,189,56,10,17>>`                                                                            |\n| `hstore`           | `%{\"foo\" => \"bar\"}`                                                                                                                         |\n| `oid types`        | `42`                                                                                                                                        |\n| `enum`             | `\"ok\"` (4)                                                                                                                                  |\n| `bit`              | `<< 1::1, 0::1 >>`                                                                                                                          |\n| `varbit`           | `<< 1::1, 0::1 >>`                                                                                                                          |\n| `tsvector`         | `[%Postgrex.Lexeme{positions: [{1, :A}], word: \"a\"}]`                                                                                       |\n\n(1) [Decimal](http://github.com/ericmj/decimal)\n\n(2) Timezones will always be normalized to UTC or assumed to be UTC when no information is available, either by PostgreSQL or Postgrex\n\n(3) `%Duration{}` may only be used with Elixir 1.17+. Intervals will only be decoded into a `%Duration{}` struct if the option `interval_decode_type: Duration` is passed to `Postgrex.Types.define/3`.\n\n(4) Enumerated types (enum) are custom named database types with strings as values.\n\n(5) Anonymous composite types are decoded (read) as tuples but they cannot be encoded (written) to the database\n\nPostgrex does not automatically cast between types. For example, you can't pass a string where a date is expected. To add type casting, support new types, or change how any of the types above are encoded/decoded, you can use extensions.","ref":"readme.html#data-representation"},{"type":"extras","title":"JSON support - Postgrex","doc":"Postgrex comes with JSON support out of the box via the [Jason](https://github.com/michalmuskala/jason) library. To use it, add :jason to your dependencies:\n\n```elixir\n{:jason, \"~> 1.0\"}\n```\n\nYou can customize it to use another library via the `:json_library` configuration:\n\n```elixir\nconfig :postgrex, :json_library, SomeOtherLib\n```\n\nOnce you change the value, you have to recompile Postgrex, which can be done by cleaning its current build:\n\n```sh\nmix deps.clean postgrex --build\n```","ref":"readme.html#json-support"},{"type":"extras","title":"Extensions - Postgrex","doc":"Extensions are used to extend Postgrex' built-in type encoding/decoding.\n\nThe [extensions](https://github.com/elixir-ecto/postgrex/blob/master/lib/postgrex/extensions/) directory in this project provides implementation for many Postgres' built-in data types. It is also a great example of how to implement your own extensions. For example, you can look at the [`Date`](https://github.com/elixir-ecto/postgrex/blob/master/lib/postgrex/extensions/date.ex) extension as a starting point.\n\nOnce you defined your extensions, you should build custom type modules, passing all of your extensions as arguments:\n\n```elixir\nPostgrex.Types.define(MyApp.PostgrexTypes, [MyApp.Postgis.Extensions], [])\n```\n\n`Postgrex.Types.define/3` must be called on its own file, outside of any module and function, as it only needs to be defined once during compilation.\n\nOnce a type module is defined, you must specify it on `start_link`:\n\n```elixir\nPostgrex.start_link(types: MyApp.PostgrexTypes)\n```","ref":"readme.html#extensions"},{"type":"extras","title":"OID type encoding - Postgrex","doc":"PostgreSQL's wire protocol supports encoding types either as text or as binary. Unlike most client libraries Postgrex uses the binary protocol, not the text protocol. This allows for efficient encoding of types (e.g. 4-byte integers are encoded as 4 bytes, not as a string of digits) and automatic support for arrays and composite types.\n\nUnfortunately the PostgreSQL binary protocol transports [OID types](http://www.postgresql.org/docs/current/static/datatype-oid.html#DATATYPE-OID-TABLE) as integers while the text protocol transports them as string of their name, if one exists, and otherwise as integer.\n\nThis means you either need to supply oid types as integers or perform an explicit cast (which would be automatic when using the text protocol) in the query.\n\n```elixir\n# Fails since $1 is regclass not text.\nquery(\"select nextval($1)\", [\"some_sequence\"])\n\n# Perform an explicit cast, this would happen automatically when using a\n# client library that uses the text protocol.\nquery(\"select nextval($1::text::regclass)\", [\"some_sequence\"])\n\n# Determine the oid once and store it for later usage. This is the most\n# efficient way, since PostgreSQL only has to perform the lookup once. Client\n# libraries using the text protocol do not support this.\n%{rows: [{sequence_oid}]} = query(\"select $1::text::regclass\", [\"some_sequence\"])\nquery(\"select nextval($1)\", [sequence_oid])\n```","ref":"readme.html#oid-type-encoding"},{"type":"extras","title":"PgBouncer - Postgrex","doc":"When using PgBouncer with transaction or statement pooling named prepared\nqueries can not be used because the bouncer may route requests from the same\npostgrex connection to different PostgreSQL backend processes and discards named\nqueries after the transactions closes. To force unnamed prepared queries:\n\n```elixir\nPostgrex.start_link(prepare: :unnamed)\n```","ref":"readme.html#pgbouncer"},{"type":"extras","title":"Contributing - Postgrex","doc":"To contribute you need to compile Postgrex from source and test it:\n\n```\n$ git clone https://github.com/elixir-ecto/postgrex.git\n$ cd postgrex\n$ mix test\n```\n\nThe tests requires some modifications to your [hba file](http://www.postgresql.org/docs/9.3/static/auth-pg-hba-conf.html). The path to it can be found by running `$ psql -U postgres -c \"SHOW hba_file\"` in your shell. Put the following above all other configurations (so that they override):\n\n```\nlocal   all             all                     trust\nhost    all             postgrex_md5_pw         127.0.0.1/32    md5\nhost    all             postgrex_cleartext_pw   127.0.0.1/32    password\nhost    all             postgrex_scram_pw       127.0.0.1/32    scram-sha-256\n```\n\nThe server needs to be restarted for the changes to take effect. Additionally you need to setup a PostgreSQL user with the same username as the local user and give it trust or ident in your hba file. Or you can export $PGUSER and $PGPASSWORD before running tests.","ref":"readme.html#contributing"},{"type":"extras","title":"Testing hstore on 9.0 - Postgrex","doc":"PostgreSQL versions 9.0 does not have the `CREATE EXTENSION` commands. This means we have to locate the postgres installation and run the `hstore.sql` in `contrib` to install `hstore`. Below is an example command to test 9.0 on OS X with homebrew installed postgres:\n\n```\n$ PGVERSION=9.0 PGPATH=/usr/local/share/postgresql9/ mix test\n```","ref":"readme.html#testing-hstore-on-9-0"},{"type":"extras","title":"License - Postgrex","doc":"Copyright 2013 Eric Meadows-Jönsson\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\n    http://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.19.0 (2024-08-03) - Changelog","doc":"* Enhancements\n  * Respect precision for interval, time, timestamp, and timestamptz\n  * Remove restriction on year 9999 on datetime columns\n  * Support decoding and encoding Elixir's v1.17 Duration as interval\n  * Allow starting one stream after the other in replication\n\n* Bug fixes\n  * Return `{:stop, state}` from `gen_statem` connection callback","ref":"changelog.html#v0-19-0-2024-08-03"},{"type":"extras","title":"v0.18.0 (2024-05-18) - Changelog","doc":"* Deprecations\n  * `:ssl_opts` is deprecated in favor of `ssl: options`\n  * `ssl: true` now emits a warning, as it does not execute server certificate verification\n\n* Enhancements\n  * Allow ReplicationConnection callbacks to trigger disconnect\n  * Add `:commit_comment` option on transactions for prepending a SQL comment to commit statements\n  * Return database messages from `handle_prepare_execute`\n\n* Backwards incompatible changes\n  * Postgrex now sets the SNI headers for SSL authentication by default. If this causes connection issues, you may set `server_name_indication: :disable` in your `:ssl_opts`","ref":"changelog.html#v0-18-0-2024-05-18"},{"type":"extras","title":"v0.17.5 (2024-03-01) - Changelog","doc":"* Enhancements\n  * Do not log if request cannot be cancelled on disconnect","ref":"changelog.html#v0-17-5-2024-03-01"},{"type":"extras","title":"v0.17.4 (2023-12-04) - Changelog","doc":"* Enhancements\n  * Trap exits from connect callback\n  * Add configuration to skip querying of `comp_oids`\n  * Handle duplicate column names in Table.Reader implementation","ref":"changelog.html#v0-17-4-2023-12-04"},{"type":"extras","title":"v0.17.3 (2023-08-29) - Changelog","doc":"* Enhancements\n  * Automatically start :ssl application\n\n* Bug fix\n  * Fix SSL configuration docs for failover case","ref":"changelog.html#v0-17-3-2023-08-29"},{"type":"extras","title":"v0.17.2 (2023-07-12) - Changelog","doc":"* Bug fix\n  * Suppress improper list warnings","ref":"changelog.html#v0-17-2-2023-07-12"},{"type":"extras","title":"v0.17.1 (2023-04-13) - Changelog","doc":"* Bug fix\n  * Fix timeout handling on SimpleConnection","ref":"changelog.html#v0-17-1-2023-04-13"},{"type":"extras","title":"v0.17.0 (2023-04-10) - Changelog","doc":"* Enhancements\n  * Add SCRAM server signature verification\n  * Support Multirange extension\n  * Support lquery and ltree extensions","ref":"changelog.html#v0-17-0-2023-04-10"},{"type":"extras","title":"v0.16.5 (2022-09-20) - Changelog","doc":"* Enhancements\n  * Allow the `:search_path` to be set for new connections","ref":"changelog.html#v0-16-5-2022-09-20"},{"type":"extras","title":"v0.16.4 (2022-07-29) - Changelog","doc":"* Enhancements\n  * Support Unix sockets in hostname and PGHOST\n  * Support infinity value on numerics/decimals (PG14+)\n  * Add count to Table.Reader metadata\n  * Fix warnings on Elixir v1.15","ref":"changelog.html#v0-16-4-2022-07-29"},{"type":"extras","title":"v0.16.3 (2022-04-27) - Changelog","doc":"* Enhancements\n  * Implement the Table.Reader protocol for query result","ref":"changelog.html#v0-16-3-2022-04-27"},{"type":"extras","title":"v0.16.2 (2022-02-21) - Changelog","doc":"* Enhancements\n  * Add `:ping_timeout` start option\n\n* Bug fixes\n  * Replication streaming can be resumed after reconnect","ref":"changelog.html#v0-16-2-2022-02-21"},{"type":"extras","title":"v0.16.1 (2022-01-24) - Changelog","doc":"* Bug fixes\n  * Fix inconsistent return type for multiple queries in `Postgrex.SimpleConnection` and `Postgrex.ReplicationConnection`. Instead always wrap `Postgrex.Result` in a list.","ref":"changelog.html#v0-16-1-2022-01-24"},{"type":"extras","title":"v0.16.0 (2022-01-23) - Changelog","doc":"Require Elixir v1.11+.\n\n* Enhancements\n  * Support negative years for date, timestamp and timestampz types\n  * Add `Postgrex.SimpleConnection` and `Postgrex.ReplicationConnection`\n* Bug fixes\n  * Cancel any pending requests before closing the socket\n  * Fix possible crash on getting default opts when PGPORT is invalid but a port is given","ref":"changelog.html#v0-16-0-2022-01-23"},{"type":"extras","title":"v0.15.11 (2021-09-26) - Changelog","doc":"* Enhancements\n  * Support `xid8` type introduced in PostgreSQL 13","ref":"changelog.html#v0-15-11-2021-09-26"},{"type":"extras","title":"v0.15.10 (2021-07-27) - Changelog","doc":"* Enhancements\n  * Define child_spec for Postgrex.Notifications\n  * Improve error handling when using multiple endpoints\n* Bug fixes\n  * Fix dialyzer warnings\n  * Fix invalid type error after failover","ref":"changelog.html#v0-15-10-2021-07-27"},{"type":"extras","title":"v0.15.9 (2021-04-24) - Changelog","doc":"* Enhancements\n  * Support the new `:endpoints` and `:target_server_type` to make it faster to rotate across multiple instances in cases of failovers\n* Bug fixes\n  * Do not warn on undefined JSON library\n  * Fix bug when a message which is not a row description, data row, command completion or error message occurs and there is buffer remaining to be processed","ref":"changelog.html#v0-15-9-2021-04-24"},{"type":"extras","title":"v0.15.8 (2021-01-19) - Changelog","doc":"* Bug fixes\n  * Make sure scram authentication method works on Erlang/OTP 24","ref":"changelog.html#v0-15-8-2021-01-19"},{"type":"extras","title":"v0.15.7 (2020-10-17) - Changelog","doc":"* Enhancements\n  * Add `compare` and `to_string` to `Postgrex.Interval`\n* Bug fixes\n  * Allow deallocated queries to be re-prepared","ref":"changelog.html#v0-15-7-2020-10-17"},{"type":"extras","title":"v0.15.6 (2020-09-21) - Changelog","doc":"* Enhancements\n  * Support Decimal 2.0\n* Bug fixes\n  * Do not keep credentials in state in `Postgrex.Notifications`","ref":"changelog.html#v0-15-6-2020-09-21"},{"type":"extras","title":"v0.15.5 (2020-06-03) - Changelog","doc":"* Enhancements\n  * Support optional decoding of infinite timestamps\n* Bug fixes\n  * Remove cache statements that cannot be described from cache when on a savepoint transaction (such as when inside the SQL sandbox)","ref":"changelog.html#v0-15-5-2020-06-03"},{"type":"extras","title":"v0.15.4 (2020-05-09) - Changelog","doc":"* Enhancements\n  * Fix warnings on Elixir v1.11.0-dev","ref":"changelog.html#v0-15-4-2020-05-09"},{"type":"extras","title":"v0.15.3 (2019-12-11) - Changelog","doc":"* Enhancements\n  * Allow dynamic connection configuration with the `:configure` for notifications\n  * Add `:auto_reconnect` option for notifications\n  * Accept `listen` commands even if the notifications connection is down or yet to first connect\n\n* Bug fixes\n  * Encode empty arrays in a mechanism compatible with CockroachDB\n  * Cleanly terminate connection started with a socket","ref":"changelog.html#v0-15-3-2019-12-11"},{"type":"extras","title":"v0.15.2 (2019-10-08) - Changelog","doc":"* Enhancements\n  * Improve performance of the bootstrap query","ref":"changelog.html#v0-15-2-2019-10-08"},{"type":"extras","title":"v0.15.1 (2019-09-16) - Changelog","doc":"* Enhancements\n  * Add support for microseconds in Postgrex.Interval\n  * Reduce bootstrap log message to debug and clarify error message","ref":"changelog.html#v0-15-1-2019-09-16"},{"type":"extras","title":"v0.15.0 (2019-07-18) - Changelog","doc":"Postgrex v0.15+ requires Elixir v1.6+.\n\n* Enhancements\n  * Filter bootstrap more efficiently by avoiding loading tables information on startup\n  * Only bootstrap new oids during describe: this means reconnects don't run a bootstrap\n    query and describe runs minimal query\n  * Parse Postgrex 12beta new version format\n  * Raise error when :ssl is required and not started in child_spec/1\n\n* Bug fixes\n  * Don't encode DateTime, NaiveDateTime or Time unless Calendar.ISO","ref":"changelog.html#v0-15-0-2019-07-18"},{"type":"extras","title":"v0.14.3 (2019-05-08) - Changelog","doc":"* Enhancements\n  * Make bootstrap query compatible with CockroachDB 19.1\n  * Improve error message when encoding bad tuple","ref":"changelog.html#v0-14-3-2019-05-08"},{"type":"extras","title":"v0.14.2 (2019-04-12) - Changelog","doc":"* Bug fixes\n  * Fix Elixir deprecation warnings\n  * Do not crash when receiving notices during authentication\n  * Do not crash when receiving an error (caused by a raise) during query execution","ref":"changelog.html#v0-14-2-2019-04-12"},{"type":"extras","title":"v0.14.1 (2018-11-24) - Changelog","doc":"* Bug fixes\n  * Bump decimal dependency to avoid runtime warnings","ref":"changelog.html#v0-14-1-2018-11-24"},{"type":"extras","title":"v0.14.0 (2018-10-29) - Changelog","doc":"* Enhancements\n  * Postgrex.INET will add a /32 netmask to an IPv4 address and a /128 netmask to an IPv6 address during encoding where `netmask: nil`. When decoding, a /32 netmask (for IPv4) or /128 netmask (for IPv6) will be removed, resulting in `netmask: nil` for the struct\n  * Add `:disconnect_on_error_codes` which allows Postgrex to automatically disconnect and then reconnect on certain errors. This is useful when using Postgrex against systems that support failover, which would emit certain errors on failover. This change allow those errors to be recovered from transparently\n  * Add `:cache_statement` to `Postgrex.query/4` as a built-in statement cache\n  * Support scram-sha-256 authentication from PostgreSQL 10\n  * Add `Postgrex.prepare_execute/4`\n  * Automatically re-prepare queries that failed to encode due to a database type change\n\n* Backwards incompatible changes\n  * Invoke `encode_to_iodata!` instead of `encode!` in JSON encoder\n  * Remove Postgrex.CIDR and use Postgrex.INET to encode both inet/cidr (as PostgreSQL may perform implicit/explicit casting at any time)\n  * Postgrex.Time, Postgrex.Date and Postgrex.Timestamp were deprecated and now have been effectively removed\n  * `Postgrex.execute/4` now always returns the prepared query\n  * `:pool_timeout` is removed in favor of `:queue_target` and `:queue_interval`. See `DBConnection.start_link/2` for more information","ref":"changelog.html#v0-14-0-2018-10-29"},{"type":"extras","title":"v0.13.4 (2018-01-25) - Changelog","doc":"* Enhancements\n  * Support custom range domains\n  * Support custom array domains\n  * Add support for UNIX domain sockets via the `:socket_dir` option\n  * Remove warnings on Elixir v1.6\n\n* Bug fixes\n  * Fix encoding of empty ranges\n  * Fix Postgrex.Path open/closed byte parity","ref":"changelog.html#v0-13-4-2018-01-25"},{"type":"extras","title":"v0.13.3 (2017-05-31) - Changelog","doc":"* Enhancements\n  * Reload types on unknown oid during prepare\n\n* Bug fixes\n  * Fix default timeout for connection process from 5000s to 15000s","ref":"changelog.html#v0-13-3-2017-05-31"},{"type":"extras","title":"v0.13.2 (2017-03-05) - Changelog","doc":"* Bug fixes\n  * Do not build invalid dates at compilation time on Elixir master","ref":"changelog.html#v0-13-2-2017-03-05"},{"type":"extras","title":"v0.13.1 (2017-02-20) - Changelog","doc":"* Enhancements\n  * Allow naming Postgrex.Notifications server\n  * Provide tsvector and lexeme support using binary formats\n\n* Bug fixes\n  * Fix encoding of Decimal values that would be wrong in certain circumstances\n  * Add `:crypto` to applications\n  * Specify proper Elixir dependency\n  * Restore compatibility with postgres versions prior to 8.4 and with redshift","ref":"changelog.html#v0-13-1-2017-02-20"},{"type":"extras","title":"v0.13.0 (2016-12-17) - Changelog","doc":"* Enhancements\n  * Support built-in geometry types\n  * Fallback to `PGDATABASE` system env for the database\n  * Support `bit` and `varbit` types\n  * Add postgres error code to error messages\n  * Support unprepared when using a stream\n  * `:connect_timeout` and `:handshake_timeout` to configure TCP connect and handshake timeouts\n  * Improve numeric encode/decode\n\n* Bug fixes\n  * Quote channel on listen/unlisten\n  * Check datetime structs available before defining calendar extension\n  * Backoff all awaiting connections if a bootstrap fails to prevent timeout loop\n  * Handle idle admin shutdown of postgres backend\n  * Fix rebootstrap query to be O(Nlog(N)) instead of O(N^2)\n  * Fix encoding of numerical values\n\n* Backwards incompatible changes\n  * `:copy_data` query option is no longer supported and data can only be copied to the database using a collectable\n  * Query struct has removed encoders/decoders and changed param_info/result_info values\n  * Extensions now use a new encoder/decoder API based on quoted expressions\n  * The `:extensions`, `:decode_binary` and `:null` options in `start_link` are no longer supported in favor of defining custom types with `Postgrex.Types.define(module, extra_extensions, options)`. `Postgrex.Types.define/3` must be called on its own file, outside of any module and function, as it only needs to be defined once during compilation.","ref":"changelog.html#v0-13-0-2016-12-17"},{"type":"extras","title":"v0.12.1 (2016-09-29) - Changelog","doc":"* Enhancements\n  * Support special \"char\" type\n\n* Bug fixes\n  * Limit re-bootstrap to one connection at a time\n  * Fix re-bootstrap of new composite types that use old types","ref":"changelog.html#v0-12-1-2016-09-29"},{"type":"extras","title":"v0.12.0 (2016-09-06) - Changelog","doc":"* Enhancements\n  * Raise DBConnection.ConnectionError on connection error\n  * Use send encoding to determine citext encoding\n  * Use Map in favor of deprecated modules (to avoid warnings on v1.4)\n  * Run rebootstrap test synchronously on every connect\n  * Add support for Elixir 1.3+ Calendar types","ref":"changelog.html#v0-12-0-2016-09-06"},{"type":"extras","title":"v0.11.2 (2016-06-16) - Changelog","doc":"* Enhancements\n  * Add support for COPY TO STDOUT and COPY FROM STDIN\n  * Support packets bigger than 64MB\n  * Introduce `mode: :savepoint` for prepare/execute/close that allows wrapping a request in a savepoint so that an error does not fail the transaction\n  * Introduce streaming queries\n  * Add `:decode_binary` option which is either `:copy` (default) or `:reference`.\n\n* Bug fixes\n  * Consistently convert the port number to integer\n  * Remove type server entry on disconnect","ref":"changelog.html#v0-11-2-2016-06-16"},{"type":"extras","title":"v0.11.1 (2016-02-15) - Changelog","doc":"* Enhancements\n  * Support PgBouncer transaction/statement pooling\n  * Include more information in error messages\n  * Add support for built-in postgres point type\n  * Add `Postgrex.child_spec/1`\n  * Allow custom encoding/decoding of postgres' NULL on a per query basis\n\n* Bug fixes\n  * Correctly pad decimal digits during encoding","ref":"changelog.html#v0-11-1-2016-02-15"},{"type":"extras","title":"v0.11.0 (2016-01-21) - Changelog","doc":"* Enhancements\n  * Rely on DBConnection. This means better performance by copying less data between processes, faster encoding/decoding, support for transactions, `after_connect` hooks, connection backoff, logging, prepared queries, the ability to use both Poolboy and Sojourn as pools out of the box, and more\n\n* Backwards incompatible change\n  * Connection API from `Postgrex.Connection` has been moved to `Postgrex`\n  * Notifications API from `Postgrex.Connection` has been moved to `Postgrex.Notifications`","ref":"changelog.html#v0-11-0-2016-01-21"},{"type":"extras","title":"v0.10.0 (2015-11-17) - Changelog","doc":"* Enhancements\n  * Improve error message on encoding/decoding failures\n  * Add network types such as: `inet`, `cidr` and `macaddr`\n  * Improve TCP error messages\n  * Support `PGPORT` environment variable\n  * Improve decoding performance by caching extension information\n  * Improve query performance by decoding in the client process hence not blocking the connection\n  * Raise if number of parameters to query is wrong\n\n* Bug fixes\n  * Correctly handle errors in connection initialization with `sync_connect: true`\n  * Do not fail on custom error codes\n  * Correctly handle large number of parameters, also fixes some protocol issues where unsigned integers were treated as signed","ref":"changelog.html#v0-10-0-2015-11-17"},{"type":"extras","title":"v0.9.1 (2015-07-14) - Changelog","doc":"* Enhancements\n  * Revert client side decoding as affects performance negatively (around 15% slower)\n  * Cast floats and integers to decimal if a decimal is requested","ref":"changelog.html#v0-9-1-2015-07-14"},{"type":"extras","title":"v0.9.0 (2015-07-12) - Changelog","doc":"* Enhancements\n  * Cached type bootstrapping for less memory usage and faster connection set up\n  * The result set is now decoded in the calling process to reduce time spent in the connection process\n  * Add a `decode: :manual` option to `Postgrex.query/4` and the function `Postgrex.decode/2` for manually decoding the result\n  * Add `:sync_connect` option to `Postgrex.start_link/1`\n\n* Bug fixes\n  * Correctly handle extension types created inside schemas\n\n* Backwards incompatible changes\n  * Each row in `Postgrex.Result.rows` is now a list of columns instead of a tuple","ref":"changelog.html#v0-9-0-2015-07-12"},{"type":"extras","title":"v0.8.4 (2015-06-24) - Changelog","doc":"* Bug fixes\n  * Fix version detection","ref":"changelog.html#v0-8-4-2015-06-24"},{"type":"extras","title":"v0.8.3 (2015-06-22) - Changelog","doc":"* Enhancements\n  * Add `Postgrex.Extensions.JSON` extension for `json` and `jsonb` types\n  * Set suitable TCP buffer size automatically","ref":"changelog.html#v0-8-3-2015-06-22"},{"type":"extras","title":"v0.8.2 (2015-06-01) - Changelog","doc":"* Enhancements\n  * Add `:socket_options` option to `Postgrex.start_link/1`\n  * Improved performance regarding binary handling\n  * Add hstore support\n\n* Backwards incompatible changes\n  * Remove `:async_connect` option and make it the default","ref":"changelog.html#v0-8-2-2015-06-01"},{"type":"extras","title":"v0.8.1 (2015-04-09) - Changelog","doc":"* Enhancements\n  * Keep the postgres error code in `:pg_code`\n  * Support oid types and all its aliases (regclass etc)\n\n* Backwards incompatible changes\n  * Rename `:msec` field to `:usec` on `Postgrex.Time` and `Postgrex.Timestamp`\n\n* Bug fixes\n  * Fix numeric encoding for fractional numbers with less digits than the numeric base\n  * Support encoding `timetz` type\n  * Fix time and timestamp off-by-one bounds","ref":"changelog.html#v0-8-1-2015-04-09"},{"type":"extras","title":"v0.8.0 (2015-02-26) - Changelog","doc":"* Enhancements\n  * Add extensions\n  * Encode/decode ranges generically\n  * Add bounds when encoding integer types to error instead of overflowing the integer\n  * Log unhandled PostgreSQL errors (when it cant be replied to anyone)\n  * Add support for enum types\n  * Add support for citext type\n  * Add microseconds to times and timestamps\n  * Add the ability to rebootstrap types for an open connection\n\n* Backwards incompatible changes\n  * Remove the support for type-hinted queries\n  * Remove encoder, decoder and formatter functions, use extensions instead\n  * Use structs for dates, times, timestamps, interval and ranges\n  * Change the default timeout for all operations to 5000ms\n  * Show PostgreSQL error codes as their names instead","ref":"changelog.html#v0-8-0-2015-02-26"},{"type":"extras","title":"v0.7.0 (2015-01-20) - Changelog","doc":"* Enhancements\n  * Add asynchronous notifications through `listen` and `unlisten`\n  * Add support for range types\n  * Add support for uuid type\n  * Add `:async_connect` option to `start_link/1`\n\n* Bug fixes\n  * Fix encoding `nil` values in arrays and composite types","ref":"changelog.html#v0-7-0-2015-01-20"},{"type":"extras","title":"v0.6.0 (2014-09-07) - Changelog","doc":"* Enhancements\n  * Queries can be constructed of `iodata`\n  * Support \"type hinted\" queries to save one client-server round trip which will reduce query latency\n\n* Backwards incompatible changes\n  * `Postgrex.Error` `postgres` field is converted from keyword list to map\n  * `Postgrex.Connect.query` `params` parameter is no longer optional (pass an empty list if query has no parameters)\n  * The `timeout` parameter for all functions have been moved to a keyword list with the key `:timeout`","ref":"changelog.html#v0-6-0-2014-09-07"},{"type":"extras","title":"v0.5.5 (2014-08-20) - Changelog","doc":"* Enhancements\n  * Reduce the amount of intermediary binaries constructed with the help of `iodata`","ref":"changelog.html#v0-5-5-2014-08-20"},{"type":"extras","title":"v0.5.4 (2014-08-04) - Changelog","doc":"","ref":"changelog.html#v0-5-4-2014-08-04"},{"type":"extras","title":"v0.5.3 (2014-07-13) - Changelog","doc":"","ref":"changelog.html#v0-5-3-2014-07-13"},{"type":"extras","title":"v0.5.2 (2014-06-18) - Changelog","doc":"","ref":"changelog.html#v0-5-2-2014-06-18"},{"type":"extras","title":"v0.5.1 (2014-05-24) - Changelog","doc":"* Backwards incompatible changes\n  * `Postgrex.Error` exception converted to struct","ref":"changelog.html#v0-5-1-2014-05-24"},{"type":"extras","title":"v0.5.0 (2014-05-01) - Changelog","doc":"* Backwards incompatible changes\n  * `Postgrex.Result` and `Postgrex.TypeInfo` converted to structs","ref":"changelog.html#v0-5-0-2014-05-01"},{"type":"extras","title":"v0.4.2 (2014-04-21) - Changelog","doc":"* Enhancements\n  * Add timeouts to all synchronous calls. When a timeout is hit an exit error will be raised in the caller process and the connection process will exit\n  * Add automatic fallback to environment variables `PGUSER`, `PGHOST` and `PGPASSWORD`","ref":"changelog.html#v0-4-2-2014-04-21"},{"type":"extras","title":"v0.4.0 (2014-01-16) - Changelog","doc":"* Enhancements\n  * Numerics decode and encode to Decimal","ref":"changelog.html#v0-4-0-2014-01-16"},{"type":"extras","title":"v0.3.1 (2014-01-15) - Changelog","doc":"* Enhancements\n  * Compact state before printing to logs and hide password\n  * Concurrency support, safe to use connection from multiple processes concurrently","ref":"changelog.html#v0-3-1-2014-01-15"},{"type":"extras","title":"v0.3.0 (2013-12-16) - Changelog","doc":"* Bug fixes\n  * Don't try to decode values of text format\n\n* Backwards incompatible changes\n  * Types are stored as binaries instead of atoms, update your custom encoders and decoders","ref":"changelog.html#v0-3-0-2013-12-16"},{"type":"extras","title":"v0.2.1 (2013-12-10) - Changelog","doc":"* Enhancements\n  * Add support for SSL\n\n* Bug fixes\n  * Fix decoding of unknown type when using custom decoder","ref":"changelog.html#v0-2-1-2013-12-10"},{"type":"extras","title":"v0.2.0 (2013-11-14) - Changelog","doc":"* Enhancements\n  * Floats handles NaN, inf and -inf\n  * Add support for numerics\n  * Custom encoders and decoders works on elements in arrays\n  * Add support for composite types\n  * Add functions that raise on error\n\n* Bug fixes\n  * INSERT query works with extended query parameters\n  * Return proper `num_rows` on PostgreSQL 8.4\n  * Fix race condition\n\n* Backwards incompatible changes\n  * Simplify custom decoding and encoding with default function","ref":"changelog.html#v0-2-0-2013-11-14"},{"type":"extras","title":"v0.1.0 (2013-10-14) - Changelog","doc":"First release!","ref":"changelog.html#v0-1-0-2013-10-14"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}